@@ -1,3 +1,4 @@
+from __future__ import print_function
 DOIT_CONFIG = {'default_tasks': ['use_cmd', 'use_python']}
 
 def task_compute():
@@ -8,18 +9,18 @@ def task_compute():
 
 def task_use_cmd():
    return {'actions': ['echo x=%(x)s, z=%(z)s'],
-           'getargs': {'x':'compute.x',
-                       'z':'compute.z'},
+           'getargs': {'x': ('compute', 'x'),
+                       'z': ('compute', 'z')},
            'verbosity': 2,
            }
 
 
 def task_use_python():
   return {'actions': [show_getargs],
-          'getargs': {'x':'compute.x',
-                      'y':'compute.y'},
+          'getargs': {'x': ('compute', 'x'),
+                      'y': ('compute', 'z')},
           'verbosity': 2,
           }
 def show_getargs(x, y):
-   print "this is x:%s" % x
-   print "this is y:%s" % y
+   print("this is x:%s" % x)
+   print("this is y:%s" % y)
