@@ -1,3 +1,4 @@
+from __future__ import print_function
 def task_compute():
    def comp(x):
        return {'x':x}
@@ -10,8 +11,8 @@ def task_compute():
 
 
 def show_getargs(values):
-    print values
-    assert sum(v['x'] for v in values) == 12
+    print(values)
+    assert sum(v['x'] for v in values.values()) == 12
 
 def task_args_dict():
   return {'actions': [show_getargs],
