@@ -2,5 +2,5 @@
 def run_once(task, values):
     def save_executed():
         return {'run-once': True}
-    task.insert_action(save_executed)
+    task.value_savers.append(save_executed)
     return values.get('run-once', False)
