@@ -6,17 +6,20 @@ import time
 import sys
 from multiprocessing import Process
 
+from .exceptions import InvalidCommand
 from .cmdparse import CmdParse
 from .filewatch import FileModifyWatcher
+from .cmd_base import tasks_and_deps_iter
 from .cmd_base import DoitCmdBase
 from .cmd_run import opt_verbosity, Run
 
-opt_reporter = {'name':'reporter',
-                 'short': None,
-                 'long': None,
-                 'type':str,
-                 'default': 'executed-only',
-                }
+opt_reporter = {
+    'name':'reporter',
+    'short': None,
+    'long': None,
+    'type':str,
+    'default': 'executed-only',
+}
 
 
 class Auto(DoitCmdBase):
@@ -31,6 +34,7 @@ class Auto(DoitCmdBase):
     doc_purpose = "automatically execute tasks when a dependency changes"
     doc_usage = "[TASK ...]"
     doc_description = None
+    execute_tasks = True
 
     cmd_options = (opt_verbosity, opt_reporter)
 
@@ -41,16 +45,9 @@ class Auto(DoitCmdBase):
         @param sel_tasks(list - str)
         """
         deps = set()
-        processed = set() # str - task name
-        to_process = set(sel_tasks) # str - task name
-        # get initial task
-        while to_process:
-            task = tasks.get(to_process.pop())
-            processed.add(task.name)
-            for task_dep in task.task_dep + task.setup_tasks:
-                if (task_dep not in processed) and (task_dep not in to_process):
-                    to_process.add(task_dep)
+        for task in tasks_and_deps_iter(tasks, sel_tasks):
             deps.update(task.file_dep)
+            deps.update(task.watch)
         return deps
 
 
@@ -75,21 +72,26 @@ class Auto(DoitCmdBase):
         started = time.time()
 
         # execute tasks using Run Command
-        ar = Run(task_loader=self._loader)
-        params.add_defaults(CmdParse(ar.options).parse([])[0])
-        result = ar.execute(params, args)
+        arun = Run(task_loader=self.loader)
+        params.add_defaults(CmdParse(arun.get_options()).parse([])[0])
+        try:
+            result = arun.execute(params, args)
+        # ??? actually tested but coverage doesnt get it...
+        except InvalidCommand as err: # pragma: no cover
+            sys.stderr.write("ERROR: %s\n" % str(err))
+            sys.exit(3)
 
         # get list of files to watch on file system
-        watch_files = self._find_file_deps(ar.control.tasks,
-                                           ar.control.selected_tasks)
+        watch_files = self._find_file_deps(arun.control.tasks,
+                                           arun.control.selected_tasks)
 
         # Check for timestamp changes since run started,
         # if change, restart straight away
-        if not self._dep_changed(watch_files, started, ar.control.targets):
+        if not self._dep_changed(watch_files, started, arun.control.targets):
             # set event handler. just terminate process.
             class DoitAutoRun(FileModifyWatcher):
                 def handle_event(self, event):
-                    #print "FS EVENT -> ", event
+                    # print("FS EVENT -> {}".format(event))
                     sys.exit(result)
             file_watcher = DoitAutoRun(watch_files)
             # kick start watching process
@@ -100,8 +102,11 @@ class Auto(DoitCmdBase):
         """loop executing tasks until process is interrupted"""
         while True:
             try:
-                p = Process(target=self.run_watch, args=(params, args))
-                p.start()
-                p.join()
+                proc = Process(target=self.run_watch, args=(params, args))
+                proc.start()
+                proc.join()
+                # if error on given command line, terminate.
+                if proc.exitcode == 3:
+                    return 3
             except KeyboardInterrupt:
                 return 0
