@@ -21,17 +21,20 @@ else:
 # note: to check which DBM backend is being used:
 #       >>> anydbm._defaultmod
 
-from .compat import json
+import json
 import sqlite3
 
 
-USE_FILE_TIMESTAMP = True
-
-
 class DatabaseException(Exception):
     """Exception class for whatever backend exception"""
     pass
 
+
+class DependencyException(Exception):
+    """Exception class for whatever backend exception"""
+    pass
+
+
 def get_md5(input_data):
     """return md5 from string or unicode"""
     if isinstance(input_data, six.text_type):
@@ -40,13 +43,14 @@ def get_md5(input_data):
         byte_data = input_data
     return hashlib.md5(byte_data).hexdigest()
 
+
 def get_file_md5(path):
     """Calculate the md5 sum from file content.
 
     @param path: (string) file path
     @return: (string) md5
     """
-    with open(path,'rb') as file_data:
+    with open(path, 'rb') as file_data:
         md5 = hashlib.md5()
         block_size = 128 * md5.block_size
         while True:
@@ -57,28 +61,6 @@ def get_file_md5(path):
     return md5.hexdigest()
 
 
-
-def check_modified(file_path, file_stat, state):
-    """check if file in file_path is modified from previous "state"
-    @param file_path (string): file path
-    @param file_stat: the value returned from os.stat(file_path)
-    @param state (tuple), timestamp, size, md5
-    @returns (bool):
-    """
-    if state is None:
-        return True
-
-    timestamp, size, file_md5 = state
-    # 1 - if timestamp is not modified file is the same
-    if USE_FILE_TIMESTAMP and file_stat.st_mtime == timestamp:
-        return False
-    # 2 - if size is different file is modified
-    if file_stat.st_size != size:
-        return True
-    # 3 - check md5
-    return file_md5 != get_file_md5(file_path)
-
-
 class JsonDB(object):
     """Backend using a single text file with JSON content"""
 
@@ -283,22 +265,25 @@ class SqliteDB(object):
         self.name = name
         self._conn = self._sqlite3(self.name)
 
-    def _sqlite3(self, name):
+    @staticmethod
+    def _sqlite3(name):
         """Open/create a sqlite3 DB file"""
         def dict_factory(cursor, row):
-            d = {}
+            """convert row to dict"""
+            data = {}
             for idx, col in enumerate(cursor.description):
-                d[col[0]] = row[idx]
-            return d
+                data[col[0]] = row[idx]
+            return data
         def converter(data):
             return json.loads(data.decode('utf-8'))
 
         sqlite3.register_adapter(list, json.dumps)
         sqlite3.register_adapter(dict, json.dumps)
         sqlite3.register_converter("json", converter)
-        conn = sqlite3.connect(self.name,
-                    detect_types=sqlite3.PARSE_DECLTYPES|sqlite3.PARSE_COLNAMES,
-                    isolation_level=None)
+        conn = sqlite3.connect(
+            name,
+            detect_types=sqlite3.PARSE_DECLTYPES|sqlite3.PARSE_COLNAMES,
+            isolation_level=None)
         conn.row_factory = dict_factory
         sqlscript = """
             create table if not exists doit (
@@ -314,7 +299,7 @@ class SqliteDB(object):
                 'To fix the issue you can just remove the database file(s) '
                 'and a new one will be generated.'
                 'Original error: %(msg)s'
-                % {'filename': repr(self.name), 'msg': str(exception)})
+                % {'filename': repr(name), 'msg': str(exception)})
             raise DatabaseException(new_message)
         return conn
 
@@ -358,9 +343,91 @@ class SqliteDB(object):
         self._conn.execute('delete from doit')
 
 
+class FileChangedChecker(object):
+    """Base checker for dependencies, must be inherited."""
 
-class DependencyBase(object):
-    """Manage tasks dependencies (abstract class)
+    def check_modified(self, file_path, file_stat, state):
+        """Check if file in file_path is modified from previous "state".
+
+        @param file_path (string): file path
+        @param file_stat: result of os.stat() of file_path
+        @param state: state that was previously saved with ``get_state()``
+        @returns (bool): True if dep is modified
+
+        """
+        raise NotImplementedError()
+
+    def get_state(self, dep, current_state):
+        """Compute the state of a task after it has been successfuly executed.
+
+        @param dep (str): path of the dependency file.
+        @param current_state (tuple): the current state, saved from a previous
+            execution of the task (None if the task was never run).
+        @returns: the new state. Return None if the state is unchanged.
+
+        The parameter `current_state` is passed to allow speed optimization,
+        see MD5Checker.get_state().
+        """
+        raise NotImplementedError()
+
+
+class MD5Checker(FileChangedChecker):
+    """MD5 checker, uses the md5sum.
+
+    This is the default checker used by doit.
+
+    As an optimization the check uses (timestamp, file-size, md5).
+    If the timestamp is the same it considers that the file has the same
+    content. If file size is different its content certainly is modified.
+    Finally the md5 is used for a different timestamp with the same size.
+    """
+
+    def check_modified(self, file_path, file_stat, state):
+        """Check if file in file_path is modified from previous "state".
+        """
+        timestamp, size, file_md5 = state
+
+        # 1 - if timestamp is not modified file is the same
+        if file_stat.st_mtime == timestamp:
+            return False
+
+        # 2 - if size is different file is modified
+        if file_stat.st_size != size:
+            return True
+
+        # 3 - check md5
+        return file_md5 != get_file_md5(file_path)
+
+
+    def get_state(self, dep, current_state):
+        timestamp = os.path.getmtime(dep)
+        # time optimization. if dep is already saved with current
+        # timestamp skip calculating md5
+        if current_state and current_state[0] == timestamp:
+            return
+        size = os.path.getsize(dep)
+        md5 = get_file_md5(dep)
+        return timestamp, size, md5
+
+
+class TimestampChecker(FileChangedChecker):
+    """Checker that use only the timestamp."""
+
+    def check_modified(self, file_path, file_stat, state):
+        return file_stat.st_mtime != state
+
+    def get_state(self, dep, current_state):
+        """@returns float: mtime for file `dep`"""
+        return os.path.getmtime(dep)
+
+
+# name of checkers class available
+CHECKERS = {'md5': MD5Checker,
+            'timestamp': TimestampChecker}
+
+
+class Dependency(object):
+    """Manage tasks dependencies
 
     Each dependency is saved in "db". the "db" can have json or dbm
     format where there is a dictionary for every task. each task has a
@@ -373,10 +440,11 @@ class DependencyBase(object):
     @ivar name: (string) filepath of the DB file
     @ivar _closed: (bool) DB was flushed to file
     """
-
-    def __init__(self, backend):
+    def __init__(self, db_class, backend_name, checker_cls=MD5Checker):
         self._closed = False
-        self.backend = backend
+        self.checker = checker_cls()
+        self.db_class = db_class
+        self.backend = db_class(backend_name)
         self._set = self.backend.set
         self._get = self.backend.get
         self.remove = self.backend.remove
@@ -393,34 +461,33 @@ class DependencyBase(object):
 
     ####### task specific
 
-    def save_success(self, task):
-        """save info after a task is successfuly executed"""
+    def save_success(self, task, result_hash=None):
+        """save info after a task is successfuly executed
+
+        :param result_hash: (str) explicitly set result_hash
+        """
         # save task values
         self._set(task.name, "_values_:", task.values)
 
         # save task result md5
-        if task.result:
+        if result_hash is not None:
+            self._set(task.name, "result:", result_hash)
+        elif task.result:
             if isinstance(task.result, dict):
                 self._set(task.name, "result:", task.result)
             else:
                 self._set(task.name, "result:", get_md5(task.result))
 
         # file-dep
+        self._set(task.name, 'checker:', self.checker.__class__.__name__)
         for dep in task.file_dep:
-            timestamp = os.path.getmtime(dep)
-            # time optimization. if dep is already saved with current timestamp
-            # skip calculating md5
-            current = self._get(task.name, dep)
-            if current and current[0] == timestamp:
-                continue
-            size = os.path.getsize(dep)
-            self._set(task.name, dep, (timestamp, size, get_file_md5(dep)))
+            state = self.checker.get_state(dep, self._get(task.name, dep))
+            if state is not None:
+                self._set(task.name, dep, state)
 
         # save list of file_deps
         self._set(task.name, 'deps:', tuple(task.file_dep))
 
-
-
     def get_values(self, task_name):
         """get all saved values from a task
         @return dict
@@ -442,6 +509,12 @@ class DependencyBase(object):
             raise Exception(msg % (task_id, key_name))
         return values[key_name]
 
+    def get_result(self, task_name):
+        """get the result saved from a task
+        @return dict or md5sum
+        """
+        return self._get(task_name, 'result:')
+
     def remove_success(self, task):
         """remove saved info from task"""
         self.remove(task.name)
@@ -458,6 +531,9 @@ class DependencyBase(object):
     def get_status(self, task, tasks_dict):
         """Check if task is up to date. set task.dep_changed
 
+        If the checker class changed since the previous run, the task is
+        deleted, to be sure that its state is not re-used.
+
         @param task: (Task)
         @param tasks_dict: (dict: Task) passed to objects used on uptodate
         @return: (str) one of up-to-date, run
@@ -470,7 +546,7 @@ class DependencyBase(object):
         task.dep_changed = []
 
         # check uptodate bool/callables
-        checked_uptodate = False
+        uptodate_result_list = []
         for utd, utd_args, utd_kwargs in task.uptodate:
             # if parameter is a callable
             if hasattr(utd, '__call__'):
@@ -509,13 +585,14 @@ class DependencyBase(object):
             # just ignored
             if uptodate_result is None:
                 continue
-            if uptodate_result:
-                checked_uptodate = True
-            else:
-                return 'run'
+            uptodate_result_list.append(uptodate_result)
+
+        # any uptodate check is false
+        if not all(uptodate_result_list):
+            return 'run'
 
         # no dependencies means it is never up to date.
-        if not (task.file_dep or checked_uptodate):
+        if not (task.file_dep or uptodate_result_list):
             return 'run'
 
         # if target file is not there, task is not up to date
@@ -524,49 +601,41 @@ class DependencyBase(object):
                 task.dep_changed = list(task.file_dep)
                 return 'run'
 
+        # check for modified file_dep checker
+        previous = self._get(task.name, 'checker:')
+        if previous and previous != self.checker.__class__.__name__:
+            task.dep_changed = list(task.file_dep)
+            # remove all saved values otherwise they might be re-used by
+            # some optmization on MD5Checker.get_state()
+            self.remove(task.name)
+            return 'run'
+
         # check for modified file_dep
-        changed = [] # list of file_dep that changed
         previous = self._get(task.name, 'deps:')
         if previous and set(previous) != task.file_dep:
-             status = 'run'
+            status = 'run'
         else:
-            status = 'up-to-date' # initial assumption
-        for dep in tuple(task.file_dep):
+            status = 'up-to-date'  # initial assumption
+
+        # list of file_dep that changed
+        check_modified = self.checker.check_modified
+        changed = []
+        for dep in task.file_dep:
+            state = self._get(task.name, dep)
             try:
                 file_stat = os.stat(dep)
-            except os.error:
-                raise Exception("Dependent file '%s' does not exist." % dep)
-            if check_modified(dep, file_stat, self._get(task.name, dep)):
+            except OSError:
+                raise DependencyException("Dependent file '{}' does not exist."
+                                          .format(dep))
+            if state is None or check_modified(dep, file_stat, state):
                 changed.append(dep)
-                status = 'run'
+        if len(changed) > 0:
+            status = 'run'
 
-        task.dep_changed = changed #FIXME create a separate function for this
+        task.dep_changed = changed  # FIXME create a separate function for this
         return status
 
 
-####################
-
-class JsonDependency(DependencyBase):
-    """Task dependency manager with JSON backend"""
-    def __init__(self, name):
-        DependencyBase.__init__(self, JsonDB(name))
-
-class DbmDependency(DependencyBase):
-    """Task dependency manager with DBM backend"""
-    def __init__(self, name):
-        DependencyBase.__init__(self, DbmDB(name))
-
-class SqliteDependency(DependencyBase):
-    """Task dependency manager with sqlite backend"""
-    def __init__(self, name):
-        DependencyBase.__init__(self, SqliteDB(name))
-
-# map string used in cmdline option to class
-backend_map = {
-    'json': JsonDependency,
-    'dbm': DbmDependency,
-    'sqlite3': SqliteDependency,
-}
 
 #############
 
@@ -581,7 +650,3 @@ class UptodateCalculator(object):
         """@param"""
         self.get_val = dep_manager._get
         self.tasks_dict = tasks_dict
-
-
-# defaut dependency backend implementation
-Dependency = DbmDependency
