@@ -3,11 +3,10 @@
 import sys
 import time
 import datetime
+import json
 import six
 from six import StringIO
 
-from .compat import json
-
 
 class ConsoleReporter(object):
     """Default reporter. print results on console/terminal (stdout/stderr)
@@ -15,6 +14,9 @@ class ConsoleReporter(object):
     @ivar show_out (bool): include captured stdout on failure report
     @ivar show_err (bool): include captured stderr on failure report
     """
+    # short description, used by the help system
+    desc = 'console output'
+
     def __init__(self, outstream, options):
         # save non-succesful result information (include task errors)
         self.failures = []
@@ -26,6 +28,10 @@ class ConsoleReporter(object):
     def write(self, text):
         self.outstream.write(text)
 
+    def initialize(self, tasks):
+        """called just after tasks have benn loaded before execution starts"""
+        pass
+
     def get_status(self, task):
         """called when task is selected (check if up-to-date)"""
         pass
@@ -79,10 +85,10 @@ class ConsoleReporter(object):
             task = result['task']
             if self.show_out:
                 out = "".join([a.out for a in task.actions if a.out])
-                self.write("%s\n" % out.encode('utf-8'))
+                self.write("%s\n" % out)
             if self.show_err:
                 err = "".join([a.err for a in task.actions if a.err])
-                self.write("%s\n" % err.encode('utf-8'))
+                self.write("%s\n" % err)
 
         if self.runtime_errors:
             self.write("#"*40 + "\n")
@@ -96,6 +102,8 @@ class ExecutedOnlyReporter(ConsoleReporter):
 
     Produces zero output unless a task is executed
     """
+    desc = 'console, no output for skipped (up-to-date) and group tasks'
+
     def skip_uptodate(self, task):
         """skipped up-to-date task"""
         pass
@@ -109,6 +117,8 @@ class ExecutedOnlyReporter(ConsoleReporter):
 
 class ZeroReporter(ConsoleReporter):
     """Report only internal errors from doit"""
+    desc = 'report only inetrnal errors from doit'
+
     def _just_pass(self, *args):
         """over-write base to do nothing"""
         pass
@@ -178,6 +188,9 @@ class JsonReporter(object):
          - started (str)
          - elapsed (float)
     """
+
+    desc = 'output in JSON format'
+
     def __init__(self, outstream, options=None): #pylint: disable=W0613
         # options parameter is not used
         # json result is sent to stdout when doit finishes running
@@ -241,18 +254,11 @@ class JsonReporter(object):
         if self.errors:
             log_err += "\n".join(self.errors)
 
-        task_result_list = [tr.to_dict() for tr in six.itervalues(self.t_results)]
+        task_result_list = [
+            tr.to_dict() for tr in six.itervalues(self.t_results)]
         json_data = {'tasks': task_result_list,
                      'out': log_out,
                      'err': log_err}
         # indent not available on simplejson 1.3 (debian etch)
         # json.dump(json_data, sys.stdout, indent=4)
         json.dump(json_data, self.outstream)
-
-
-# name of reporters class available to be selected on cmd line
-REPORTERS = {'default': ConsoleReporter,
-             'executed-only': ExecutedOnlyReporter,
-             'json': JsonReporter,
-             'zero': ZeroReporter,
-             }
