@@ -3,7 +3,7 @@ from collections import deque
 import pytest
 
 from doit.exceptions import InvalidDodoFile, InvalidCommand
-from doit.task import InvalidTask, Task
+from doit.task import InvalidTask, Task, DelayedLoader
 from doit.control import TaskControl, TaskDispatcher, ExecNode
 from doit.control import no_none
 
@@ -101,6 +101,16 @@ class TestTaskControlCmdOptions(object):
         tc =  TaskControl(tasks)
         assert ['tX'] == tc._filter_tasks(["targetX"])
 
+    def test_filter_delayed_subtask(self):
+        t1 = Task("taskX", None)
+        t2 = Task("taskY", None, loader=DelayedLoader(lambda: None))
+        control = TaskControl([t1, t2])
+        control._filter_tasks(['taskY:foo'])
+        assert isinstance(t2.loader, DelayedLoader)
+        # sub-task will use same loader, and keep parent basename
+        assert control.tasks['taskY:foo'].loader.basename == 'taskY'
+        assert control.tasks['taskY:foo'].loader is t2.loader
+
     # filter a non-existent task raises an error
     def testFilterWrongName(self):
         tc =  TaskControl(TASKS_SAMPLE)
@@ -117,6 +127,14 @@ class TestTaskControlCmdOptions(object):
         assert ['t3', 't1'] == tc._filter_tasks(options)
         assert "hello option!" == tc.tasks['t3'].options['opt1']
 
+    def testPosParam(self):
+        tasks = list(TASKS_SAMPLE)
+        tasks.append(Task("tP", [""],[],[], pos_arg='myp'))
+        tc = TaskControl(tasks)
+        args = ["tP", "hello option!", "t1"]
+        assert ['tP',] == tc._filter_tasks(args)
+        assert ["hello option!", "t1"] == tc.tasks['tP'].pos_arg_val
+
 
 class TestExecNode(object):
     def test_repr(self):
@@ -232,6 +250,22 @@ class TestTaskDispatcher_node_add_wait_run(object):
         td._node_add_wait_run(n1, ['t2'])
         assert n1.bad_deps
 
+    def test_calc_dep_already_executed(self):
+        tasks = {'t1': Task('t1', None, calc_dep=['t2']),
+                 't2': Task('t2', None),
+                 }
+        td = TaskDispatcher(tasks, [], None)
+        n1 = td._gen_node(None, 't1')
+        n2 = td._gen_node(None, 't2')
+        n2.run_status = 'done'
+        n2.task.values = {'calc_dep': ['t3'], 'task_dep':['t5']}
+        td._node_add_wait_run(n1, ['t2'], calc=True)
+        # n1 is updated with results from t2
+        assert n1.calc_dep == set(['t2', 't3'])
+        assert n1.task_dep == ['t5']
+        # n1 doesnt need to wait any calc_dep to be executed
+        assert n1.wait_run_calc == set()
+
 
 class TestTaskDispatcher_add_task(object):
     def test_no_deps(self):
@@ -312,6 +346,22 @@ class TestTaskDispatcher_add_task(object):
         assert tasks['t1'] == n1.step()
 
 
+    def test_calc_dep_already_executed(self):
+        tasks = {'t1': Task('t1', None, calc_dep=['t2']),
+                 't2': Task('t2', None),
+                 't3': Task('t3', None),
+                 }
+        td = TaskDispatcher(tasks, {'intermediate': 't3'}, None)
+        n1 = td._gen_node(None, 't1')
+        n2 = td._gen_node(None, 't2')
+        n2.run_status = 'done'
+        n2.task.values = {'calc_dep': ['t3']}
+        assert 't3' == n1.step().task.name
+        assert set() == n1.wait_run
+        assert set() == n1.wait_run_calc
+        #assert False
+
+
     def test_setup_task__run(self):
         tasks = {'t1': Task('t1', None, setup=['t2']),
                  't2': Task('t2', None),
@@ -327,6 +377,87 @@ class TestTaskDispatcher_add_task(object):
         assert tasks['t1'] == next(gen)  # second time
 
 
+    def test_delayed_creation(self):
+        def creator():
+            yield Task('foo', None, loader=DelayedLoader(lambda : None))
+        delayed_loader = DelayedLoader(creator, executed='t2')
+        tasks = {'t1': Task('t1', None, loader=delayed_loader),
+                 't2': Task('t2', None),
+                 }
+        td = TaskDispatcher(tasks, [], None)
+        n1 = td._gen_node(None, 't1')
+        gen = td._add_task(n1)
+
+        # first returned node is `t2` because it is an implicit task_dep
+        n2 = next(gen)
+        assert n2.task.name == 't2'
+
+        # wait until t2 is finished
+        n3 = next(gen)
+        assert n3 == 'wait'
+
+        # after t2 is done, generator is reseted
+        td._update_waiting(n2)
+        n4 = next(gen)
+        assert n4 == "reset generator"
+
+        # recursive loader is preserved
+        assert isinstance(td.tasks['foo'].loader, DelayedLoader)
+        pytest.raises(AssertionError, next, gen)
+
+
+    def test_delayed_creation_sub_task(self):
+        # usually a repeated loader is replaced by the real task
+        # when it is first executed, the problem arises when the
+        # the expected task is not actually created
+        def creator():
+            yield Task('t1:foo', None)
+            yield Task('t1:bar', None)
+        delayed_loader = DelayedLoader(creator, executed='t2')
+        tasks = {
+            't1': Task('t1', None, loader=delayed_loader),
+            't2': Task('t2', None),}
+
+        # simulate a sub-task from delayed created added to task_list
+        tasks['t1:foo'] = Task('t1:foo', None, loader=delayed_loader)
+        tasks['t1:xxx'] = Task('t1:xxx', None, loader=delayed_loader)
+        delayed_loader.basename = 't1'
+
+        td = TaskDispatcher(tasks, [], None)
+        n1 = td._gen_node(None, 't1:foo')
+        gen = td._add_task(n1)
+
+        # first returned node is `t2` because it is an implicit task_dep
+        n1b = next(gen)
+        assert n1b.task.name == 't2'
+
+        # wait until t2 is finished
+        n1c = next(gen)
+        assert n1c == 'wait'
+
+        # after t2 is done, generator is reseted
+        n1b.run_status = 'successful'
+        td._update_waiting(n1b)
+        n1d = next(gen)
+        assert n1d == "reset generator"
+        assert 't1:foo' in td.tasks
+        assert 't1:bar' in td.tasks
+
+        # finish with t1:foo
+        gen2 = td._add_task(n1)
+        n1.reset_task(td.tasks[n1.task.name], gen2)
+        n2 = next(gen2)
+        assert n2.name == 't1:foo'
+        pytest.raises(StopIteration, next, gen2)
+
+        # try non-existent t1:xxx
+        n3 = td._gen_node(None, 't1:xxx')
+        gen3 = td._add_task(n3)
+        # ? should raise a runtime error?
+        pytest.raises(StopIteration, next, gen3)
+
+
+
 class TestTaskDispatcher_get_next_node(object):
     def test_none(self):
         tasks = {'t1': Task('t1', None, task_dep=['t2']),
@@ -411,7 +542,30 @@ class TestTaskDispatcher_update_waiting(object):
         assert deque([n1]) == td.ready
         assert 0 == len(td.waiting)
 
-    # test_wait_calc is tested on TestTaskDispatcher_add_task.test_calc_dep
+    def test_waiting_node_updated(self):
+        tasks = {'t1': Task('t1', None, calc_dep=['t2'], task_dep=['t4']),
+                 't2': Task('t2', None),
+                 't3': Task('t3', None),
+                 't4': Task('t4', None),
+                 }
+        td = TaskDispatcher(tasks, [], None)
+        n1 = td._gen_node(None, 't1')
+        n1_gen = td._add_task(n1)
+        n2 = next(n1_gen)
+        assert 't2' == n2.task.name
+        assert 't4' == next(n1_gen).task.name
+        assert 'wait' == next(n1_gen)
+        assert set() == n1.calc_dep
+        assert td.waiting == set()
+
+        n2.run_status = 'done'
+        n2.task.values = {'calc_dep': ['t2', 't3'], 'task_dep':['t5']}
+        assert n1.calc_dep == set()
+        assert n1.task_dep == []
+        td._update_waiting(n2)
+        assert n1.calc_dep == set(['t3'])
+        assert n1.task_dep == ['t5']
+
 
 
 class TestTaskDispatcher_dispatcher_generator(object):
@@ -427,3 +581,40 @@ class TestTaskDispatcher_dispatcher_generator(object):
         assert "hold on" == next(gen) # hold until t2 is done
         assert tasks[0] == gen.send(n2).task
         pytest.raises(StopIteration, lambda gen: next(gen), gen)
+
+
+    def test_delayed_creation(self):
+        def creator():
+            yield {'name': 'foo1', 'actions': None, 'file_dep': ['bar']}
+            yield {'name': 'foo2', 'actions': None, 'targets': ['bar']}
+
+        delayed_loader = DelayedLoader(creator, executed='t2')
+        tasks = [Task('t0', None, task_dep=['t1']),
+                 Task('t1', None, loader=delayed_loader),
+                 Task('t2', None)]
+
+        control = TaskControl(tasks)
+        control.process(['t0'])
+        disp = control.task_dispatcher()
+        gen = disp.generator
+        nt2 = next(gen)
+        assert nt2.task.name == "t2"
+
+        # wait for t2 to be executed
+        assert "hold on" == next(gen)
+        assert "hold on" == next(gen) # hold until t2 is done
+
+        # delayed creation of tasks for t1 does not mess existing info
+        assert disp.nodes['t1'].waiting_me == set([disp.nodes['t0']])
+        nf2 = gen.send(nt2)
+        assert disp.nodes['t1'].waiting_me == set([disp.nodes['t0']])
+
+        assert nf2.task.name == "t1:foo2"
+        nf1 = gen.send(nf2)
+        assert nf1.task.name == "t1:foo1"
+        assert nf1.task.task_dep == ['t1:foo2'] # implicit dep added
+        nt1 = gen.send(nf1)
+        assert nt1.task.name == "t1"
+        nt0 = gen.send(nt1)
+        assert nt0.task.name == "t0"
+        pytest.raises(StopIteration, lambda gen: next(gen), gen)
