@@ -7,11 +7,16 @@ import six
 import pytest
 
 from doit.task import Task
-from doit.dependency import get_md5, md5sum, check_modified, UptodateCalculator
-from doit.dependency import DbmDB, DatabaseException
-from doit.dependency import JsonDependency, DbmDependency, SqliteDependency
+from doit.dependency import get_md5, get_file_md5
+from doit.dependency import DbmDB, JsonDB, SqliteDB, Dependency
+from doit.dependency import DatabaseException, UptodateCalculator
+from doit.dependency import FileChangedChecker, MD5Checker, TimestampChecker
 from .conftest import get_abspath, depfile
 
+#path to test folder
+TEST_PATH = os.path.dirname(__file__)
+PROGRAM = "python %s/sample_process.py" % TEST_PATH
+
 
 def test_unicode_md5():
     data = six.u("我")
@@ -20,18 +25,17 @@ def test_unicode_md5():
 
 
 def test_md5():
-    filePath = os.path.join(os.path.dirname(__file__),"sample_md5.txt")
+    filePath = os.path.join(os.path.dirname(__file__), "sample_md5.txt")
     # result got using command line md5sum
     expected = "45d1503cb985898ab5bd8e58973007dd"
-    assert expected == md5sum(filePath)
+    assert expected == get_file_md5(filePath)
 
 
 ####
-# dependencies are files only (not other tasks), or bool.
+# dependencies are files only (not other tasks).
 #
 # whenever a task has a dependency the runner checks if this dependency
 # was modified since last successful run. if not the task is skipped.
-# if depedency is a bool. it is always up-to-date if present.
 
 # since more than one task might have the same dependency, and the tasks
 # might have different results (success/failure). the signature is associated
@@ -49,7 +53,7 @@ def test_md5():
 @pytest.fixture
 def pdepfile(request):
     return depfile(request)
-pytest.fixture(params=[JsonDependency, DbmDependency, SqliteDependency])(pdepfile)
+pytest.fixture(params=[JsonDB, DbmDB, SqliteDB])(pdepfile)
 
 # FIXME there was major refactor breaking classes from dependency,
 # unit-tests could be more specific to base classes.
@@ -73,13 +77,13 @@ class TestDependencyDb(object):
         pdepfile.close()
 
         # open it again and check the value
-        d2 = pdepfile.__class__(pdepfile.name)
+        d2 = Dependency(pdepfile.db_class, pdepfile.name)
 
         value = d2._get("taskId_X","dependency_A")
         assert "da_md5" == value, value
 
     def test_corrupted_file(self, pdepfile):
-        if pdepfile.__class__==DbmDependency and pdepfile.whichdb is None: # pragma: no cover
+        if pdepfile.whichdb is None: # pragma: no cover
             pytest.skip('dumbdbm too dumb to detect db corruption')
 
         # create some corrupted files
@@ -88,11 +92,12 @@ class TestDependencyDb(object):
             fd = open(full_name, 'w')
             fd.write("""{"x": y}""")
             fd.close()
-        pytest.raises(DatabaseException, pdepfile.__class__, pdepfile.name)
+        pytest.raises(DatabaseException, Dependency,
+                      pdepfile.db_class, pdepfile.name)
 
     def test_corrupted_file_unrecognized_excep(self, monkeypatch, pdepfile):
-        if isinstance(pdepfile, JsonDependency):
-            pytest.skip('test doesnt apply to JsonDependency')
+        if pdepfile.db_class is not DbmDB:
+            pytest.skip('test doesnt apply to non DBM DB')
         if pdepfile.whichdb is None: # pragma: no cover
             pytest.skip('dumbdbm too dumb to detect db corruption')
 
@@ -103,7 +108,8 @@ class TestDependencyDb(object):
             fd.write("""{"x": y}""")
             fd.close()
         monkeypatch.setattr(DbmDB, 'DBM_CONTENT_ERROR_MSG', 'xxx')
-        pytest.raises(DatabaseException, pdepfile.__class__, pdepfile.name)
+        pytest.raises(DatabaseException, Dependency,
+                      pdepfile.db_class, pdepfile.name)
 
     # _get must return None if entry doesnt exist.
     def test_getNonExistent(self, pdepfile):
@@ -133,11 +139,11 @@ class TestDependencyDb(object):
         pdepfile._set("taskId_YYY", "dep_1", "x")
         pdepfile.close()
         # 2 - re-open and remove one task
-        reopened = pdepfile.__class__(pdepfile.name)
+        reopened = Dependency(pdepfile.db_class, pdepfile.name)
         reopened.remove("taskId_YYY")
         reopened.close()
         # 3 - re-open again and check task was really removed
-        reopened2 = pdepfile.__class__(pdepfile.name)
+        reopened2 = Dependency(pdepfile.db_class, pdepfile.name)
         assert reopened2._in("taskId_XXX")
         assert not reopened2._in("taskId_YYY")
 
@@ -152,7 +158,6 @@ class TestDependencyDb(object):
         assert None == pdepfile._get("taskId_YYY","dep_1")
 
 
-
 class TestSaveSuccess(object):
 
     def test_save_result(self, pdepfile):
@@ -160,6 +165,13 @@ class TestSaveSuccess(object):
         t1.result = "result"
         pdepfile.save_success(t1)
         assert get_md5("result") == pdepfile._get(t1.name, "result:")
+        assert get_md5("result") == pdepfile.get_result(t1.name)
+
+    def test_save_result_hash(self, pdepfile):
+        t1 = Task('t_name', None)
+        t1.result = "result"
+        pdepfile.save_success(t1, result_hash='abc')
+        assert 'abc' == pdepfile._get(t1.name, "result:")
 
     def test_save_resultNone(self, pdepfile):
         t1 = Task('t_name', None)
@@ -215,6 +227,7 @@ class TestSaveSuccess(object):
         pdepfile.save_success(t1)
         assert pdepfile._get("taskId_X",filePath) is not None
         assert pdepfile._get("taskId_X",filePath2) is not None
+        assert set(pdepfile._get("taskId_X", 'deps:')) == t1.file_dep
 
     def test_save_values(self, pdepfile):
         t1 = Task('t1', None)
@@ -255,7 +268,6 @@ class TestGetValue(object):
         pytest.raises(Exception, pdepfile.get_value, 't1', 'z')
 
 
-
 class TestRemoveSuccess(object):
     def test_save_result(self, pdepfile):
         t1 = Task('t_name', None)
@@ -273,27 +285,54 @@ class TestIgnore(object):
         assert '1' == pdepfile._get(t1.name, "ignore:")
 
 
-class TestCheckModified(object):
-    def test_None(self, dependency1):
-        assert check_modified(dependency1, os.stat(dependency1),  None)
-
+class TestMD5Checker(object):
     def test_timestamp(self, dependency1):
-        timestamp = os.path.getmtime(dependency1)
-        dep_stat = os.stat(dependency1)
-        assert not check_modified(dependency1, dep_stat, (timestamp, 0, ''))
-        assert check_modified(dependency1, dep_stat, (timestamp+1, 0, ''))
+        checker = MD5Checker()
+        state = checker.get_state(dependency1, None)
+        state2 = (state[0], state[1]+1, '')
+        file_stat = os.stat(dependency1)
+        # dep considered the same as long as timestamp is unchanged
+        assert not checker.check_modified(dependency1, file_stat, state2)
 
-    def test_size_md5(self, dependency1):
-        timestamp = os.path.getmtime(dependency1)
-        size = os.path.getsize(dependency1)
-        md5 = md5sum(dependency1)
-        dep_stat = os.stat(dependency1)
-        # incorrect size dont check md5
-        assert check_modified(dependency1, dep_stat, (timestamp+1, size+1, ''))
-        # correct size check md5
-        assert not check_modified(dependency1, dep_stat, (timestamp+1, size, md5))
-        assert check_modified(dependency1, dep_stat, (timestamp+1, size, ''))
+    def test_size(self, dependency1):
+        checker = MD5Checker()
+        state = checker.get_state(dependency1, None)
+        state2 = (state[0]+1, state[1]+1, state[2])
+        file_stat = os.stat(dependency1)
+        # if size changed for sure modified (md5 is not checked)
+        assert checker.check_modified(dependency1, file_stat, state2)
 
+    def test_md5(self, dependency1):
+        checker = MD5Checker()
+        state = checker.get_state(dependency1, None)
+        file_stat = os.stat(dependency1)
+        # same size and md5
+        state2 = (state[0]+1, state[1], state[2])
+        assert not checker.check_modified(dependency1, file_stat, state2)
+        # same size, different md5
+        state3 = (state[0]+1, state[1], 'not me')
+        assert checker.check_modified(dependency1, file_stat, state3)
+
+
+class TestCustomChecker(object):
+
+    def test_not_implemented(self, dependency1):
+        class MyChecker(FileChangedChecker):
+            pass
+
+        checker = MyChecker()
+        pytest.raises(NotImplementedError, checker.get_state, None, None)
+        pytest.raises(NotImplementedError, checker.check_modified,
+                      None, None, None)
+
+
+class TestTimestampChecker(object):
+    def test_timestamp(self, dependency1):
+        checker = TimestampChecker()
+        state = checker.get_state(dependency1, None)
+        file_stat = os.stat(dependency1)
+        assert not checker.check_modified(dependency1, file_stat, state)
+        assert checker.check_modified(dependency1, file_stat, state+1)
 
 
 class TestGetStatus(object):
@@ -336,6 +375,35 @@ class TestGetStatus(object):
         assert 'run' == pdepfile.get_status(t1, {})
         assert dependencies == t1.dep_changed
 
+    def test_fileDependencies_changed(self, pdepfile):
+        filePath = get_abspath("data/dependency1")
+        ff = open(filePath,"w")
+        ff.write("part1")
+        ff.close()
+
+        filePath2 = get_abspath("data/dependency2")
+        ff = open(filePath,"w")
+        ff.write("part1")
+        ff.close()
+
+        dependencies = [filePath, filePath2]
+        t1 = Task("t1", None, dependencies)
+
+        # first time execute
+        assert 'run' == pdepfile.get_status(t1, {})
+        assert sorted(dependencies) == sorted(t1.dep_changed)
+
+        # second time no
+        pdepfile.save_success(t1)
+        assert 'up-to-date' == pdepfile.get_status(t1, {})
+        assert [] == t1.dep_changed
+
+        # remove dependency filePath2
+        t1 = Task("t1", None, [filePath])
+        # execute again
+        assert 'run' == pdepfile.get_status(t1, {})
+        assert [] == t1.dep_changed
+
 
     def test_file_dependency_not_exist(self, pdepfile):
         filePath = get_abspath("data/dependency_not_exist")
@@ -343,6 +411,18 @@ class TestGetStatus(object):
         pytest.raises(Exception, pdepfile.get_status, t1, {})
 
 
+    def test_change_checker(self, pdepfile, dependency1):
+        t1 = Task("taskId_X", None, [dependency1])
+        pdepfile.checker = TimestampChecker()
+        pdepfile.save_success(t1)
+        assert 'up-to-date' == pdepfile.get_status(t1, {})
+        # change of checker force `run` again
+        pdepfile.checker = MD5Checker()
+        assert 'run' == pdepfile.get_status(t1, {})
+        pdepfile.save_success(t1)
+        assert 'up-to-date' == pdepfile.get_status(t1, {})
+
+
     # if there is no dependency the task is always executed
     def test_noDependency(self, pdepfile):
         t1 = Task("t1", None)
@@ -394,13 +474,15 @@ class TestGetStatus(object):
         assert 'up-to-date' == pdepfile.get_status(t1, {})
 
 
-    def test_UptodateCallable_True(self, pdepfile):
-        def check(task, values): return True
+    def test_UptodateFunction_True(self, pdepfile):
+        def check(task, values):
+            assert task.name == 't1'
+            return True
         t1 = Task("t1", None, uptodate=[check])
         pdepfile.save_success(t1)
         assert 'up-to-date' == pdepfile.get_status(t1, {})
 
-    def test_UptodateCallable_False(self, pdepfile):
+    def test_UptodateFunction_False(self, pdepfile):
         filePath = get_abspath("data/dependency1")
         ff = open(filePath,"w")
         ff.write("part1")
@@ -418,6 +500,48 @@ class TestGetStatus(object):
         assert 'run' == pdepfile.get_status(t1, {})
         assert [] == t1.dep_changed
 
+    def test_UptodateFunction_without_args_True(self, pdepfile):
+        def check(): return True
+        t1 = Task("t1", None, uptodate=[check])
+        pdepfile.save_success(t1)
+        assert 'up-to-date' == pdepfile.get_status(t1, {})
+
+    def test_uptodate_call_all_even_if_some_False(self, pdepfile):
+        checks = []
+        def check():
+            checks.append(1)
+            return False
+        t1 = Task("t1", None, uptodate=[check, check])
+        #pdepfile.save_success(t1)
+        assert 'run' == pdepfile.get_status(t1, {})
+        assert 2 == len(checks)
+
+
+    def test_UptodateFunction_extra_args_True(self, pdepfile):
+        def check(task, values, control):
+            assert task.name == 't1'
+            return control>30
+        t1 = Task("t1", None, uptodate=[ (check, [34]) ])
+        pdepfile.save_success(t1)
+        assert 'up-to-date' == pdepfile.get_status(t1, {})
+
+    def test_UptodateCallable_True(self, pdepfile):
+        class MyChecker(object):
+            def __call__(self, task, values):
+                assert task.name == 't1'
+                return True
+        t1 = Task("t1", None, uptodate=[ MyChecker() ])
+        pdepfile.save_success(t1)
+        assert 'up-to-date' == pdepfile.get_status(t1, {})
+
+    def test_UptodateMethod_True(self, pdepfile):
+        class MyChecker(object):
+            def check(self, task, values):
+                assert task.name == 't1'
+                return True
+        t1 = Task("t1", None, uptodate=[ MyChecker().check ])
+        pdepfile.save_success(t1)
+        assert 'up-to-date' == pdepfile.get_status(t1, {})
 
     def test_UptodateCallable_added_attributes(self, pdepfile):
         task_dict = "fake dict"
@@ -432,6 +556,16 @@ class TestGetStatus(object):
         t1 = Task("t1", None, uptodate=[check])
         assert 'up-to-date' == pdepfile.get_status(t1, task_dict)
 
+    def test_UptodateCommand_True(self, pdepfile):
+        t1 = Task("t1", None, uptodate=[PROGRAM])
+        pdepfile.save_success(t1)
+        assert 'up-to-date' == pdepfile.get_status(t1, {})
+
+    def test_UptodateCommand_False(self, pdepfile):
+        t1 = Task("t1", None, uptodate=[PROGRAM + ' please fail'])
+        pdepfile.save_success(t1)
+        assert 'run' == pdepfile.get_status(t1, {})
+
 
     # if target file does not exist, task is outdated.
     def test_targets_notThere(self, pdepfile, dependency1):
