@@ -1,22 +1,24 @@
+import os
+
 import pytest
 from mock import Mock
 
 from doit import get_var
 from doit.exceptions import InvalidCommand
-from doit.doit_cmd import DoitMain
 from doit.cmd_run import Run
 from doit.cmd_list import List
+from doit import doit_cmd
+
 
 
 def cmd_main(args):
-    return DoitMain().run(args)
+    return doit_cmd.DoitMain().run(args)
 
 
 class TestRun(object):
     def test_version(self, capsys):
         cmd_main(["--version"])
         out, err = capsys.readouterr()
-        assert "bin" in out
         assert "lib" in out
 
     def test_usage(self, capsys):
@@ -24,26 +26,6 @@ class TestRun(object):
         out, err = capsys.readouterr()
         assert "doit list" in out
 
-    def test_help_usage(self, capsys):
-        cmd_main(["help"])
-        out, err = capsys.readouterr()
-        assert "doit list" in out
-
-    def test_help_task(self, capsys):
-        cmd_main(["help", "task"])
-        out, err = capsys.readouterr()
-        assert "Task Dictionary parameters" in out
-
-    def test_help_cmd(self, capsys):
-        cmd_main(["help", "list"])
-        out, err = capsys.readouterr()
-        assert "Purpose: list tasks from dodo file" in out
-
-    def test_help_wrong_name(self, capsys):
-        cmd_main(["help", "wrong_name"])
-        out, err = capsys.readouterr()
-        assert "doit list" in out
-
     def test_run_is_default(self, monkeypatch):
         mock_run = Mock()
         monkeypatch.setattr(Run, "execute", mock_run)
@@ -69,6 +51,26 @@ class TestRun(object):
         cmd_main(['--z=5'])
         assert None == get_var('--z')
 
+    def test_task_loader_has_cmd_list(self, monkeypatch):
+        cmd_names = []
+        def save_cmd_names(self, params, args):
+            cmd_names.extend(self.loader.cmd_names)
+        monkeypatch.setattr(Run, "execute", save_cmd_names)
+        cmd_main([])
+        assert 'list' in cmd_names
+
+    def test_extra_config(self, monkeypatch, depfile_name):
+        outfile_val = []
+        def monkey_run(self, opt_values, pos_args):
+            outfile_val.append(opt_values['outfile'])
+        monkeypatch.setattr(Run, "execute", monkey_run)
+        extra_config = {
+            'outfile': 'foo.txt',
+            'dep_file': depfile_name,
+        }
+        doit_cmd.DoitMain(extra_config={'GLOBAL': extra_config}).run([])
+        assert outfile_val[0] == 'foo.txt'
+
 
 
 class TestErrors(object):
@@ -96,3 +98,33 @@ class TestErrors(object):
         # traceback from Exception (this case code from mock lib)
         assert "mock.py" in err
 
+
+class TestConfig(object):
+    def test_no_ini_config_file(self):
+        main = doit_cmd.DoitMain(config_filenames=())
+        main.run(['--version'])
+
+    def test_load_plugins_command(self):
+        config_filename = os.path.join(os.path.dirname(__file__), 'sample.cfg')
+        main = doit_cmd.DoitMain(config_filenames=config_filename)
+        assert 1 == len(main.config['COMMAND'])
+        # test loaded plugin command is actually used with plugin name
+        assert 'foo' in main.get_cmds()
+
+    def test_merge_api_ini_config(self):
+        config_filename = os.path.join(os.path.dirname(__file__), 'sample.cfg')
+        api_config = {'GLOBAL': {'opty':'10', 'optz':'10'}}
+        main = doit_cmd.DoitMain(config_filenames=config_filename,
+                                 extra_config=api_config)
+        assert 1 == len(main.config['COMMAND'])
+        # test loaded plugin command is actually used with plugin name
+        assert 'foo' in main.get_cmds()
+        # INI has higher preference the api_config
+        assert main.config['GLOBAL'] == {'optx':'6', 'opty':'7', 'optz':'10'}
+
+    def test_execute_command_plugin(self, capsys):
+        config_filename = os.path.join(os.path.dirname(__file__), 'sample.cfg')
+        main = doit_cmd.DoitMain(config_filenames=config_filename)
+        main.run(['foo'])
+        got = capsys.readouterr()[0]
+        assert got == 'this command does nothing!\n'
