@@ -1,9 +1,11 @@
 import os
+import inspect
 
 import pytest
 
+import doit
 from doit.exceptions import InvalidDodoFile, InvalidCommand
-from doit.task import InvalidTask, Task
+from doit.task import InvalidTask, DelayedLoader, Task
 from doit.loader import flat_generator, get_module
 from doit.loader import load_tasks, load_doit_config, generate_tasks
 
@@ -18,12 +20,12 @@ class TestFlatGenerator(object):
 
 
 class TestGetModule(object):
-    def testAbsolutePath(self, cwd):
+    def testAbsolutePath(self, restore_cwd):
         fileName = os.path.join(os.path.dirname(__file__),"loader_sample.py")
         dodo_module = get_module(fileName)
         assert hasattr(dodo_module, 'task_xxx1')
 
-    def testRelativePath(self, cwd):
+    def testRelativePath(self, restore_cwd):
         # test relative import but test should still work from any path
         # so change cwd.
         this_path = os.path.join(os.path.dirname(__file__),'..')
@@ -36,26 +38,29 @@ class TestGetModule(object):
         fileName = os.path.join(os.path.dirname(__file__),"i_dont_exist.py")
         pytest.raises(InvalidDodoFile, get_module, fileName)
 
-    def testInParentDir(self, cwd):
-        os.chdir('data')
+    def testInParentDir(self, restore_cwd):
+        os.chdir(os.path.join(os.path.dirname(__file__), "data"))
         fileName = "loader_sample.py"
         pytest.raises(InvalidDodoFile, get_module, fileName)
         get_module(fileName, seek_parent=True)
         # cwd is changed to location of dodo.py
         assert os.getcwd() == os.path.dirname(os.path.abspath(fileName))
 
-    def testWrongFileNameInParentDir(self, cwd):
-        os.chdir('data')
+    def testWrongFileNameInParentDir(self, restore_cwd):
+        os.chdir(os.path.join(os.path.dirname(__file__), "data"))
         fileName = os.path.join("i_dont_exist.py")
         pytest.raises(InvalidDodoFile, get_module, fileName, seek_parent=True)
 
-    def testSetCwd(self, cwd):
+    def testSetCwd(self, restore_cwd):
+        initial_wd = os.getcwd()
         fileName = os.path.join(os.path.dirname(__file__),"loader_sample.py")
         cwd = os.path.join(os.path.dirname(__file__), "data")
+        assert cwd != initial_wd # make sure test is not too easy
         get_module(fileName, cwd)
         assert os.getcwd() == cwd, os.getcwd()
+        assert doit.get_initial_workdir() == initial_wd
 
-    def testInvalidCwd(self, cwd):
+    def testInvalidCwd(self, restore_cwd):
         fileName = os.path.join(os.path.dirname(__file__),"loader_sample.py")
         cwd = os.path.join(os.path.dirname(__file__), "dataX")
         pytest.raises(InvalidCommand, get_module, fileName, cwd)
@@ -83,6 +88,27 @@ class TestLoadTasks(object):
         assert 'xxx1' == task_list[0].name
         assert 'yyy2' == task_list[1].name
 
+    def testCreateAfterDecorator(self):
+        @doit.create_after('yyy2')
+        def task_zzz3(): # pragma: no cover
+            pass
+
+        # create_after annotates the function
+        assert isinstance(task_zzz3.doit_create_after, DelayedLoader)
+        assert task_zzz3.doit_create_after.task_dep == 'yyy2'
+
+    def testInitialLoadDelayedTask(self, dodo):
+        @doit.create_after('yyy2')
+        def task_zzz3(): # pragma: no cover
+            raise Exception('Cant be executed on load phase')
+        dodo['task_zzz3'] = task_zzz3
+
+        # placeholder task is created with `loader` attribute
+        task_list = load_tasks(dodo, allow_delayed=True)
+        z_task = [t for t in task_list if t.name=='zzz3'][0]
+        assert z_task.loader.task_dep == 'yyy2'
+        assert z_task.loader.creator == task_zzz3
+
     def testNameInBlacklist(self):
         dodo_module = {'task_cmd_name': lambda:None}
         pytest.raises(InvalidDodoFile, load_tasks, dodo_module, ['cmd_name'])
@@ -109,6 +135,23 @@ class TestLoadTasks(object):
         assert len(task_list) == 1
         assert task_list[0].file_dep == set(['fooy'])
 
+    def testUse_object_methods(self):
+        class Dodo(object):
+            def foo(self): # pragma: no cover
+                pass
+
+            def task_method1(self):
+                return {'actions':None}
+
+            def task_method2(self):
+                return {'actions':None}
+
+        methods = dict(inspect.getmembers(Dodo()))
+        task_list = load_tasks(methods)
+        assert 2 == len(task_list)
+        assert 'method1' == task_list[0].name
+        assert 'method2' == task_list[1].name
+
 
 class TestDodoConfig(object):
 
@@ -135,12 +178,17 @@ class TestGenerateTaskNone(object):
         assert len(tasks) == 0
 
 
-class TestGenerateTasksDict(object):
+class TestGenerateTasksSingle(object):
     def testDict(self):
         tasks = generate_tasks("my_name", {'actions':['xpto 14']})
         assert isinstance(tasks[0], Task)
         assert "my_name" == tasks[0].name
 
+    def testTaskObj(self):
+        tasks = generate_tasks("foo", Task('bar', None))
+        assert 1 == len(tasks)
+        assert tasks[0].name == 'bar'
+
     def testBaseName(self):
         tasks = generate_tasks("function_name", {
                 'basename': 'real_task_name',
@@ -171,7 +219,7 @@ class TestGenerateTasksGenerator(object):
         def f_xpto():
             for i in range(3):
                 yield {'name':str(i), 'actions' :["xpto -%d"%i]}
-        tasks = sorted(generate_tasks("xpto", f_xpto()))
+        tasks = generate_tasks("xpto", f_xpto())
         assert isinstance(tasks[0], Task)
         assert 4 == len(tasks)
         assert not tasks[0].is_subtask
@@ -188,7 +236,7 @@ class TestGenerateTasksGenerator(object):
         def f_first_level():
             for i in range(2):
                 yield f_xpto(str(i))
-        tasks = sorted(generate_tasks("xpto", f_first_level()))
+        tasks = generate_tasks("xpto", f_first_level())
         assert isinstance(tasks[0], Task)
         assert 7 == len(tasks)
         assert not tasks[0].is_subtask
@@ -198,6 +246,18 @@ class TestGenerateTasksGenerator(object):
         assert "xpto:1-2" == tasks[-1].name
 
 
+    def testGeneratorReturnTaskObj(self):
+        def foo(base_name):
+            for i in range(3):
+                name = "%s-%d" % (base_name, i)
+                yield Task(name, actions=["xpto -%d"%i])
+        tasks = generate_tasks("foo", foo('bar'))
+        assert 3 == len(tasks)
+        assert tasks[0].name == 'bar-0'
+        assert tasks[1].name == 'bar-1'
+        assert tasks[2].name == 'bar-2'
+
+
     def testGeneratorDoesntReturnDict(self):
         def f_xpto():
             for i in range(3):
