@@ -1,16 +1,22 @@
 # coding=UTF-8
 
 import sys
+import json
 from six import StringIO
 
 from doit import reporter
 from doit.task import Task
 from doit.exceptions import CatchedException
-from doit.compat import json
 
 
 class TestConsoleReporter(object):
 
+    def test_initialize(self):
+        rep = reporter.ConsoleReporter(StringIO(), {})
+        rep.initialize([Task("t_name", None)])
+        # no output on initialize
+        assert "" in rep.outstream.getvalue()
+
     def test_startTask(self):
         rep = reporter.ConsoleReporter(StringIO(), {})
         rep.get_status(Task("t_name", None))
