@@ -5,8 +5,9 @@ import six
 import pytest
 from mock import Mock
 
-from doit.dependency import Dependency
-from doit.task import Task
+from doit.exceptions import InvalidTask
+from doit.dependency import DbmDB, Dependency
+from doit.task import Task, DelayedLoader
 from doit.control import TaskDispatcher, ExecNode
 from doit import runner
 
@@ -20,7 +21,8 @@ def _error():
     raise Exception("I am the exception.\n")
 def _exit():
     raise SystemExit()
-
+def simple_result():
+    return 'my-result'
 
 class FakeReporter(object):
     """Just log everything in internal attribute - used on tests"""
@@ -64,65 +66,65 @@ def reporter(request):
 
 
 class TestRunner(object):
-    def testInit(self, reporter, depfile):
-        my_runner = runner.Runner(Dependency, depfile.name, reporter)
+    def testInit(self, reporter, dep_manager):
+        my_runner = runner.Runner(dep_manager, reporter)
         assert False == my_runner._stop_running
         assert runner.SUCCESS == my_runner.final_result
 
 
 class TestRunner_SelectTask(object):
-    def test_ready(self, reporter, depfile):
+    def test_ready(self, reporter, dep_manager):
         t1 = Task("taskX", [(my_print, ["out a"] )])
-        my_runner = runner.Runner(Dependency, depfile.name, reporter)
+        my_runner = runner.Runner(dep_manager, reporter)
         assert True == my_runner.select_task(ExecNode(t1, None), {})
         assert ('start', t1) == reporter.log.pop(0)
         assert not reporter.log
 
-    def test_DependencyError(self, reporter, depfile):
+    def test_DependencyError(self, reporter, dep_manager):
         t1 = Task("taskX", [(my_print, ["out a"] )],
                   file_dep=["i_dont_exist"])
-        my_runner = runner.Runner(Dependency, depfile.name, reporter)
+        my_runner = runner.Runner(dep_manager, reporter)
         assert False == my_runner.select_task(ExecNode(t1, None), {})
         assert ('start', t1) == reporter.log.pop(0)
         assert ('fail', t1) == reporter.log.pop(0)
         assert not reporter.log
 
-    def test_upToDate(self, reporter, depfile):
+    def test_upToDate(self, reporter, dep_manager):
         t1 = Task("taskX", [(my_print, ["out a"] )], file_dep=[__file__])
-        my_runner = runner.Runner(Dependency, depfile.name, reporter)
+        my_runner = runner.Runner(dep_manager, reporter)
         my_runner.dep_manager.save_success(t1)
         assert False == my_runner.select_task(ExecNode(t1, None), {})
         assert ('start', t1) == reporter.log.pop(0)
         assert ('up-to-date', t1) == reporter.log.pop(0)
         assert not reporter.log
 
-    def test_ignore(self, reporter, depfile):
+    def test_ignore(self, reporter, dep_manager):
         t1 = Task("taskX", [(my_print, ["out a"] )])
-        my_runner = runner.Runner(Dependency, depfile.name, reporter)
+        my_runner = runner.Runner(dep_manager, reporter)
         my_runner.dep_manager.ignore(t1)
         assert False == my_runner.select_task(ExecNode(t1, None), {})
         assert ('start', t1) == reporter.log.pop(0)
         assert ('ignore', t1) == reporter.log.pop(0)
         assert not reporter.log
 
-    def test_alwaysExecute(self, reporter, depfile):
+    def test_alwaysExecute(self, reporter, dep_manager):
         t1 = Task("taskX", [(my_print, ["out a"] )])
-        my_runner = runner.Runner(Dependency, depfile.name, reporter, always_execute=True)
+        my_runner = runner.Runner(dep_manager, reporter, always_execute=True)
         my_runner.dep_manager.save_success(t1)
         assert True == my_runner.select_task(ExecNode(t1, None), {})
         assert ('start', t1) == reporter.log.pop(0)
         assert not reporter.log
 
-    def test_noSetup_ok(self, reporter, depfile):
+    def test_noSetup_ok(self, reporter, dep_manager):
         t1 = Task("taskX", [(my_print, ["out a"] )])
-        my_runner = runner.Runner(Dependency, depfile.name, reporter)
+        my_runner = runner.Runner(dep_manager, reporter)
         assert True == my_runner.select_task(ExecNode(t1, None), {})
         assert ('start', t1) == reporter.log.pop(0)
         assert not reporter.log
 
-    def test_withSetup(self, reporter, depfile):
+    def test_withSetup(self, reporter, dep_manager):
         t1 = Task("taskX", [(my_print, ["out a"] )], setup=["taskY"])
-        my_runner = runner.Runner(Dependency, depfile.name, reporter)
+        my_runner = runner.Runner(dep_manager, reporter)
         # defer execution
         n1 = ExecNode(t1, None)
         assert False == my_runner.select_task(n1, {})
@@ -133,7 +135,7 @@ class TestRunner_SelectTask(object):
         assert not reporter.log
 
 
-    def test_getargs_ok(self, reporter, depfile):
+    def test_getargs_ok(self, reporter, dep_manager):
         def ok(): return {'x':1}
         def check_x(my_x): return my_x == 1
         t1 = Task('t1', [(ok,)])
@@ -141,7 +143,7 @@ class TestRunner_SelectTask(object):
         t2 = Task('t2', [(check_x,)], getargs={'my_x':('t1','x')})
         n2 = ExecNode(t2, None)
         tasks_dict = {'t1': t1, 't2':t2}
-        my_runner = runner.Runner(Dependency, depfile.name, reporter)
+        my_runner = runner.Runner(dep_manager, reporter)
 
         # t2 gives chance for setup tasks to be executed
         assert False == my_runner.select_task(n2, tasks_dict)
@@ -161,7 +163,7 @@ class TestRunner_SelectTask(object):
         assert not reporter.log
         assert {'my_x': 1} == t2.options
 
-    def test_getargs_fail(self, reporter, depfile):
+    def test_getargs_fail(self, reporter, dep_manager):
         # invalid getargs. Exception wil be raised and task will fail
         def check_x(my_x): return True
         t1 = Task('t1', [lambda :True])
@@ -169,7 +171,7 @@ class TestRunner_SelectTask(object):
         t2 = Task('t2', [(check_x,)], getargs={'my_x':('t1','x')})
         n2 = ExecNode(t2, None)
         tasks_dict = {'t1': t1, 't2':t2}
-        my_runner = runner.Runner(Dependency, depfile.name, reporter)
+        my_runner = runner.Runner(dep_manager, reporter)
 
         # t2 gives chance for setup tasks to be executed
         assert False == my_runner.select_task(n2, tasks_dict)
@@ -189,13 +191,13 @@ class TestRunner_SelectTask(object):
         assert not reporter.log
 
 
-    def test_getargs_dict(self, reporter, depfile):
+    def test_getargs_dict(self, reporter, dep_manager):
         def ok(): return {'x':1}
         t1 = Task('t1', [(ok,)])
         n1 = ExecNode(t1, None)
         t2 = Task('t2', None, getargs={'my_x':('t1', None)})
         tasks_dict = {'t1': t1, 't2':t2}
-        my_runner = runner.Runner(Dependency, depfile.name, reporter)
+        my_runner = runner.Runner(dep_manager, reporter)
         t1_result = my_runner.execute_task(t1)
         my_runner.process_task_result(n1, t1_result)
 
@@ -205,13 +207,13 @@ class TestRunner_SelectTask(object):
         assert {'my_x': {'x':1}} == t2.options
 
 
-    def test_getargs_group(self, reporter, depfile):
+    def test_getargs_group(self, reporter, dep_manager):
         def ok(): return {'x':1}
         t1 = Task('t1', None, task_dep=['t1:a'], has_subtask=True)
         t1a = Task('t1:a', [(ok,)], is_subtask=True)
         t2 = Task('t2', None, getargs={'my_x':('t1', None)})
         tasks_dict = {'t1': t1, 't1a':t1a, 't2':t2}
-        my_runner = runner.Runner(Dependency, depfile.name, reporter)
+        my_runner = runner.Runner(dep_manager, reporter)
         t1a_result = my_runner.execute_task(t1a)
         my_runner.process_task_result(ExecNode(t1a, None), t1a_result)
 
@@ -222,13 +224,13 @@ class TestRunner_SelectTask(object):
 
 
 
-    def test_getargs_group_value(self, reporter, depfile):
+    def test_getargs_group_value(self, reporter, dep_manager):
         def ok(): return {'x':1}
         t1 = Task('t1', None, task_dep=['t1:a'], has_subtask=True)
         t1a = Task('t1:a', [(ok,)], is_subtask=True)
         t2 = Task('t2', None, getargs={'my_x':('t1', 'x')})
         tasks_dict = {'t1': t1, 't1a':t1a, 't2':t2}
-        my_runner = runner.Runner(Dependency, depfile.name, reporter)
+        my_runner = runner.Runner(dep_manager, reporter)
         t1a_result = my_runner.execute_task(t1a)
         my_runner.process_task_result(ExecNode(t1a, None), t1a_result)
 
@@ -240,35 +242,35 @@ class TestRunner_SelectTask(object):
 
 
 class TestTask_Teardown(object):
-    def test_ok(self, reporter, depfile):
+    def test_ok(self, reporter, dep_manager):
         touched = []
         def touch():
             touched.append(1)
         t1 = Task('t1', [], teardown=[(touch,)])
-        my_runner = runner.Runner(Dependency, depfile.name, reporter)
+        my_runner = runner.Runner(dep_manager, reporter)
         my_runner.teardown_list = [t1]
         my_runner.teardown()
         assert 1 == len(touched)
         assert ('teardown', t1) == reporter.log.pop(0)
         assert not reporter.log
 
-    def test_reverse_order(self, reporter, depfile):
+    def test_reverse_order(self, reporter, dep_manager):
         def do_nothing():pass
         t1 = Task('t1', [], teardown=[do_nothing])
         t2 = Task('t2', [], teardown=[do_nothing])
-        my_runner = runner.Runner(Dependency, depfile.name, reporter)
+        my_runner = runner.Runner(dep_manager, reporter)
         my_runner.teardown_list = [t1, t2]
         my_runner.teardown()
         assert ('teardown', t2) == reporter.log.pop(0)
         assert ('teardown', t1) == reporter.log.pop(0)
         assert not reporter.log
 
-    def test_errors(self, reporter, depfile):
+    def test_errors(self, reporter, dep_manager):
         def raise_something(x):
             raise Exception(x)
         t1 = Task('t1', [], teardown=[(raise_something,['t1 blow'])])
         t2 = Task('t2', [], teardown=[(raise_something,['t2 blow'])])
-        my_runner = runner.Runner(Dependency, depfile.name, reporter)
+        my_runner = runner.Runner(dep_manager, reporter)
         my_runner.teardown_list = [t1, t2]
         my_runner.teardown()
         assert ('teardown', t2) == reporter.log.pop(0)
@@ -279,11 +281,12 @@ class TestTask_Teardown(object):
 
 
 class TestTask_RunAll(object):
-    def test_reporter_runtime_error(self, reporter, depfile):
+    def test_reporter_runtime_error(self, reporter, dep_manager):
         t1 = Task('t1', [], calc_dep=['t2'])
         t2 = Task('t2', [lambda: {'file_dep':[1]}])
-        my_runner = runner.Runner(Dependency, depfile.name, reporter)
+        my_runner = runner.Runner(dep_manager, reporter)
         my_runner.run_all(TaskDispatcher({'t1':t1, 't2':t2}, [], ['t1', 't2']))
+        assert runner.ERROR == my_runner.final_result
         assert ('start', t2) == reporter.log.pop(0)
         assert ('execute', t2) == reporter.log.pop(0)
         assert ('success', t2) == reporter.log.pop(0)
@@ -306,20 +309,20 @@ def ok2(): return "different"
 
 class TestRunner_run_tasks(object):
 
-    def test_teardown(self, reporter, RunnerClass, depfile):
+    def test_teardown(self, reporter, RunnerClass, dep_manager):
         t1 = Task('t1', [], teardown=[ok])
         t2 = Task('t2', [])
-        my_runner = RunnerClass(Dependency, depfile.name, reporter)
+        my_runner = RunnerClass(dep_manager, reporter)
         assert [] == my_runner.teardown_list
         my_runner.run_tasks(TaskDispatcher({'t1':t1, 't2':t2}, [], ['t1', 't2']))
         my_runner.finish()
         assert ('teardown', t1) == reporter.log[-1]
 
     # testing whole process/API
-    def test_success(self, reporter, RunnerClass, depfile):
+    def test_success(self, reporter, RunnerClass, dep_manager):
         t1 = Task("t1", [(my_print, ["out a"] )] )
         t2 = Task("t2", [(my_print, ["out a"] )] )
-        my_runner = RunnerClass(Dependency, depfile.name, reporter)
+        my_runner = RunnerClass(dep_manager, reporter)
         my_runner.run_tasks(TaskDispatcher({'t1':t1, 't2':t2}, [], ['t1', 't2']))
         assert runner.SUCCESS == my_runner.finish()
         assert ('start', t1) == reporter.log.pop(0), reporter.log
@@ -330,14 +333,14 @@ class TestRunner_run_tasks(object):
         assert ('success', t2) == reporter.log.pop(0)
 
     # test result, value, out, err are saved into task
-    def test_result(self, reporter, RunnerClass, depfile):
+    def test_result(self, reporter, RunnerClass, dep_manager):
         def my_action():
             import sys
             sys.stdout.write('out here')
             sys.stderr.write('err here')
             return {'bb': 5}
         task = Task("taskY", [my_action] )
-        my_runner = RunnerClass(Dependency, depfile.name, reporter)
+        my_runner = RunnerClass(dep_manager, reporter)
         assert None == task.result
         assert {} == task.values
         assert [None] == [a.out for a in task.actions]
@@ -350,10 +353,10 @@ class TestRunner_run_tasks(object):
         assert ['err here'] == [a.err for a in task.actions]
 
     # whenever a task fails remaining task are not executed
-    def test_failureOutput(self, reporter, RunnerClass, depfile):
+    def test_failureOutput(self, reporter, RunnerClass, dep_manager):
         t1 = Task("t1", [_fail])
         t2 = Task("t2", [_fail])
-        my_runner = RunnerClass(Dependency, depfile.name, reporter)
+        my_runner = RunnerClass(dep_manager, reporter)
         my_runner.run_tasks(TaskDispatcher({'t1':t1, 't2':t2}, [], ['t1', 't2']))
         assert runner.FAILURE == my_runner.finish()
         assert ('start', t1) == reporter.log.pop(0)
@@ -363,10 +366,10 @@ class TestRunner_run_tasks(object):
         assert 0 == len(reporter.log)
 
 
-    def test_error(self, reporter, RunnerClass, depfile):
+    def test_error(self, reporter, RunnerClass, dep_manager):
         t1 = Task("t1", [_error])
         t2 = Task("t2", [_error])
-        my_runner = RunnerClass(Dependency, depfile.name, reporter)
+        my_runner = RunnerClass(dep_manager, reporter)
         my_runner.run_tasks(TaskDispatcher({'t1':t1, 't2':t2}, [], ['t1', 't2']))
         assert runner.ERROR == my_runner.finish()
         assert ('start', t1) == reporter.log.pop(0)
@@ -377,7 +380,7 @@ class TestRunner_run_tasks(object):
 
 
     # when successful dependencies are updated
-    def test_updateDependencies(self, reporter, RunnerClass, depfile):
+    def test_updateDependencies(self, reporter, RunnerClass, depfile_name):
         depPath = os.path.join(os.path.dirname(__file__),"data/dependency1")
         ff = open(depPath,"a")
         ff.write("xxx")
@@ -391,18 +394,19 @@ class TestRunner_run_tasks(object):
         targets = [filePath]
 
         t1 = Task("t1", [my_print], dependencies, targets)
-        my_runner = RunnerClass(Dependency, depfile.name, reporter)
+        dep_manager = Dependency(DbmDB, depfile_name)
+        my_runner = RunnerClass(dep_manager, reporter)
         my_runner.run_tasks(TaskDispatcher({'t1':t1}, [], ['t1']))
         assert runner.SUCCESS == my_runner.finish()
-        d = Dependency(depfile.name)
+        d = Dependency(DbmDB, depfile_name)
         assert d._get("t1", os.path.abspath(depPath))
 
 
-    def test_continue(self, reporter, RunnerClass, depfile):
+    def test_continue(self, reporter, RunnerClass, dep_manager):
         t1 = Task("t1", [(_fail,)] )
         t2 = Task("t2", [(_error,)] )
         t3 = Task("t3", [(ok,)])
-        my_runner = RunnerClass(Dependency, depfile.name, reporter, continue_=True)
+        my_runner = RunnerClass(dep_manager, reporter, continue_=True)
         disp = TaskDispatcher({'t1':t1, 't2':t2, 't3':t3}, [], ['t1', 't2', 't3'])
         my_runner.run_tasks(disp)
         assert runner.ERROR == my_runner.finish()
@@ -419,11 +423,11 @@ class TestRunner_run_tasks(object):
 
 
     def test_continue_dont_execute_parent_of_failed_task(self, reporter,
-                                                         RunnerClass, depfile):
+                                                         RunnerClass, dep_manager):
         t1 = Task("t1", [(_error,)] )
         t2 = Task("t2", [(ok,)], task_dep=['t1'])
         t3 = Task("t3", [(ok,)])
-        my_runner = RunnerClass(Dependency, depfile.name, reporter, continue_=True)
+        my_runner = RunnerClass(dep_manager, reporter, continue_=True)
         disp = TaskDispatcher({'t1':t1, 't2':t2, 't3':t3}, [], ['t1', 't2', 't3'])
         my_runner.run_tasks(disp)
         assert runner.ERROR == my_runner.finish()
@@ -438,10 +442,10 @@ class TestRunner_run_tasks(object):
         assert 0 == len(reporter.log)
 
 
-    def test_continue_dep_error(self, reporter, RunnerClass, depfile):
+    def test_continue_dep_error(self, reporter, RunnerClass, dep_manager):
         t1 = Task("t1", [(ok,)], file_dep=['i_dont_exist'] )
         t2 = Task("t2", [(ok,)], task_dep=['t1'])
-        my_runner = RunnerClass(Dependency, depfile.name, reporter, continue_=True)
+        my_runner = RunnerClass(dep_manager, reporter, continue_=True)
         disp = TaskDispatcher({'t1':t1, 't2':t2}, [], ['t1', 't2'])
         my_runner.run_tasks(disp)
         assert runner.ERROR == my_runner.finish()
@@ -452,10 +456,10 @@ class TestRunner_run_tasks(object):
         assert 0 == len(reporter.log)
 
 
-    def test_continue_ignored_dep(self, reporter, RunnerClass, depfile):
+    def test_continue_ignored_dep(self, reporter, RunnerClass, dep_manager):
         t1 = Task("t1", [(ok,)], )
         t2 = Task("t2", [(ok,)], task_dep=['t1'])
-        my_runner = RunnerClass(Dependency, depfile.name, reporter, continue_=True)
+        my_runner = RunnerClass(dep_manager, reporter, continue_=True)
         my_runner.dep_manager.ignore(t1)
         disp = TaskDispatcher({'t1':t1, 't2':t2}, [], ['t1', 't2'])
         my_runner.run_tasks(disp)
@@ -467,13 +471,13 @@ class TestRunner_run_tasks(object):
         assert 0 == len(reporter.log)
 
 
-    def test_getargs(self, reporter, RunnerClass, depfile):
+    def test_getargs(self, reporter, RunnerClass, dep_manager):
         def use_args(arg1):
             six.print_(arg1)
         def make_args(): return {'myarg':1}
         t1 = Task("t1", [(use_args,)], getargs=dict(arg1=('t2','myarg')) )
         t2 = Task("t2", [(make_args,)])
-        my_runner = RunnerClass(Dependency, depfile.name, reporter)
+        my_runner = RunnerClass(dep_manager, reporter)
         my_runner.run_tasks(TaskDispatcher({'t1':t1, 't2':t2}, [], ['t1', 't2']))
         assert runner.SUCCESS == my_runner.finish()
         assert ('start', t1) == reporter.log.pop(0)
@@ -485,10 +489,23 @@ class TestRunner_run_tasks(object):
         assert 0 == len(reporter.log)
 
 
+    def testActionModifiesFiledep(self, reporter, RunnerClass, dep_manager):
+        extra_dep = os.path.join(os.path.dirname(__file__), 'sample_md5.txt')
+        def action_add_filedep(task):
+            task.file_dep.add(extra_dep)
+        t1 = Task("t1", [(my_print, ["out a"] ), action_add_filedep] )
+        my_runner = RunnerClass(dep_manager, reporter)
+        my_runner.run_tasks(TaskDispatcher({'t1':t1}, [], ['t1']))
+        assert runner.SUCCESS == my_runner.finish()
+        assert ('start', t1) == reporter.log.pop(0), reporter.log
+        assert ('execute', t1) == reporter.log.pop(0)
+        assert ('success', t1) == reporter.log.pop(0)
+        assert t1.file_dep == set([extra_dep])
+
     # SystemExit runner should not interfere with SystemExit
-    def testSystemExitRaises(self, reporter, RunnerClass, depfile):
+    def testSystemExitRaises(self, reporter, RunnerClass, dep_manager):
         t1 = Task("t1", [_exit])
-        my_runner = RunnerClass(Dependency, depfile.name, reporter)
+        my_runner = RunnerClass(dep_manager, reporter)
         disp = TaskDispatcher({'t1':t1}, [], ['t1'])
         pytest.raises(SystemExit, my_runner.run_tasks, disp)
         my_runner.finish()
@@ -516,73 +533,103 @@ class TestMReporter(object):
         assert not hasattr(mp_reporter, 'no_existent_method')
 
 
+class TestJobTask(object):
+    def test_not_picklable(self):
+        def non_top_function(): pass
+        t1 = Task('t1', [non_top_function])
+        pytest.raises(InvalidTask, runner.JobTask, t1)
+
+
 @pytest.mark.skipif('not runner.MRunner.available()')
-class TestMRunner_get_next_task(object):
+class TestMRunner_get_next_job(object):
     # simple normal case
-    def test_run_task(self, reporter, depfile):
+    def test_run_task(self, reporter, dep_manager):
         t1 = Task('t1', [])
         t2 = Task('t2', [])
-        run = runner.MRunner(Dependency, depfile.name, reporter)
+        run = runner.MRunner(dep_manager, reporter)
         run._run_tasks_init(TaskDispatcher({'t1':t1, 't2':t2}, [], ['t1', 't2']))
-        assert t1 == run.get_next_task(None).task
-        assert t2 == run.get_next_task(None).task
-        assert None == run.get_next_task(None)
+        assert t1.name == run.get_next_job(None).name
+        assert t2.name == run.get_next_job(None).name
+        assert None == run.get_next_job(None)
 
-    def test_stop_running(self, reporter, depfile):
+    def test_stop_running(self, reporter, dep_manager):
         t1 = Task('t1', [])
         t2 = Task('t2', [])
-        run = runner.MRunner(Dependency, depfile.name, reporter)
+        run = runner.MRunner(dep_manager, reporter)
         run._run_tasks_init(TaskDispatcher({'t1':t1, 't2':t2}, [], ['t1', 't2']))
-        assert t1 == run.get_next_task(None).task
+        assert t1.name == run.get_next_job(None).name
         run._stop_running = True
-        assert None == run.get_next_task(None)
+        assert None == run.get_next_job(None)
 
-    def test_waiting(self, reporter, depfile):
+    def test_waiting(self, reporter, dep_manager):
         t1 = Task('t1', [])
         t2 = Task('t2', [], setup=('t1',))
-        run = runner.MRunner(Dependency, depfile.name, reporter)
-        run._run_tasks_init(TaskDispatcher({'t1':t1, 't2':t2}, [], ['t2']))
+        run = runner.MRunner(dep_manager, reporter)
+        dispatcher = TaskDispatcher({'t1':t1, 't2':t2}, [], ['t2'])
+        run._run_tasks_init(dispatcher)
 
         # first start task 1
-        n1 = run.get_next_task(None)
-        assert t1 == n1.task
+        j1 = run.get_next_job(None)
+        assert t1.name == j1.name
 
         # hold until t1 is done
-        assert isinstance(run.get_next_task(None), runner.Hold)
-        assert isinstance(run.get_next_task(None), runner.Hold)
+        assert isinstance(run.get_next_job(None), runner.JobHold)
+        assert isinstance(run.get_next_job(None), runner.JobHold)
+
+        n1 = dispatcher.nodes[j1.name]
         n1.run_status = 'done'
-
-        n2 = run.get_next_task(n1)
-        assert t2 == n2.task
-        assert None == run.get_next_task(n2)
+        j2 = run.get_next_job(n1)
+        assert t2.name == j2.name
+        assert None == run.get_next_job(dispatcher.nodes[j2.name])
 
 
-    def test_waiting_controller(self, reporter, depfile):
+    def test_waiting_controller(self, reporter, dep_manager):
         t1 = Task('t1', [])
         t2 = Task('t2', [], calc_dep=('t1',))
-        run = runner.MRunner(Dependency, depfile.name, reporter)
+        run = runner.MRunner(dep_manager, reporter)
         run._run_tasks_init(TaskDispatcher({'t1':t1, 't2':t2}, [], ['t1', 't2']))
 
         # first task ok
-        assert t1 == run.get_next_task(None).task
+        assert t1.name == run.get_next_job(None).name
 
         # hold until t1 finishes
         assert 0 == run.free_proc
-        assert isinstance(run.get_next_task(None), runner.Hold)
+        assert isinstance(run.get_next_job(None), runner.JobHold)
         assert 1 == run.free_proc
 
 
+    def test_delayed_loaded(self, reporter, dep_manager):
+        def create():
+            return {'basename':'t1', 'actions': None}
+        t1 = Task('t1', [], loader=DelayedLoader(create, executed='t2'))
+        t2 = Task('t2', [])
+        run = runner.MRunner(dep_manager, reporter)
+        dispatcher = TaskDispatcher({'t1':t1, 't2':t2}, [], ['t1', 't2'])
+        run._run_tasks_init(dispatcher)
+        assert t2.name == run.get_next_job(None).name
+        assert runner.JobHold.type == run.get_next_job(None).type
+
+        # after t2 is done t1 can be dispatched
+        n2 = dispatcher.nodes[t2.name]
+        n2.run_status = 'done'
+        j1 = run.get_next_job(n2)
+        assert t1.name == j1.name
+        # the job for t1 contains the whole task since sub-process dont
+        # have it
+        assert j1.type == runner.JobTask.type
+
+
 
 @pytest.mark.skipif('not runner.MRunner.available()')
 class TestMRunner_start_process(object):
     # 2 process, 3 tasks
-    def test_all_processes(self, reporter, monkeypatch, depfile):
+    def test_all_processes(self, reporter, monkeypatch, dep_manager):
         mock_process = Mock()
         monkeypatch.setattr(runner.MRunner, 'Child', mock_process)
         t1 = Task('t1', [])
         t2 = Task('t2', [])
         td = TaskDispatcher({'t1':t1, 't2':t2}, [], ['t1', 't2'])
-        run = runner.MRunner(Dependency, depfile.name, reporter, num_process=2)
+        run = runner.MRunner(dep_manager, reporter, num_process=2)
         run._run_tasks_init(td)
         result_q = Queue()
         task_q = Queue()
@@ -595,12 +642,12 @@ class TestMRunner_start_process(object):
 
 
     # 2 process, 1 task
-    def test_less_processes(self, reporter, monkeypatch, depfile):
+    def test_less_processes(self, reporter, monkeypatch, dep_manager):
         mock_process = Mock()
         monkeypatch.setattr(runner.MRunner, 'Child', mock_process)
         t1 = Task('t1', [])
         td = TaskDispatcher({'t1':t1}, [], ['t1'])
-        run = runner.MRunner(Dependency, depfile.name, reporter, num_process=2)
+        run = runner.MRunner(dep_manager, reporter, num_process=2)
         run._run_tasks_init(td)
         result_q = Queue()
         task_q = Queue()
@@ -612,13 +659,13 @@ class TestMRunner_start_process(object):
 
 
     # 2 process, 2 tasks (but only one task can be started)
-    def test_waiting_process(self, reporter, monkeypatch, depfile):
+    def test_waiting_process(self, reporter, monkeypatch, dep_manager):
         mock_process = Mock()
         monkeypatch.setattr(runner.MRunner, 'Child', mock_process)
         t1 = Task('t1', [])
         t2 = Task('t2', [], task_dep=['t1'])
         td = TaskDispatcher({'t1':t1, 't2':t2}, [], ['t1', 't2'])
-        run = runner.MRunner(Dependency, depfile.name, reporter, num_process=2)
+        run = runner.MRunner(dep_manager, reporter, num_process=2)
         run._run_tasks_init(td)
         result_q = Queue()
         task_q = Queue()
@@ -627,21 +674,67 @@ class TestMRunner_start_process(object):
         run.finish()
         assert 2 == len(proc_list)
         assert t1.name == task_q.get().name
-        assert isinstance(task_q.get(), runner.Hold)
+        assert isinstance(task_q.get(), runner.JobHold)
+
+
+class TestMRunner_parallel_run_tasks(object):
+
+    @pytest.mark.skipif('not runner.MRunner.available()')
+    def test_task_not_picklabe_multiprocess(self, reporter, dep_manager):
+        def creator():
+            return {'basename': 't2', 'actions': [lambda: 5]}
+        t1 = Task("t1", [(my_print, ["out a"] )] )
+        t2 = Task("t2", None, loader=DelayedLoader(creator, executed='t1'))
+        my_runner = runner.MRunner(dep_manager, reporter)
+        dispatcher = TaskDispatcher({'t1':t1, 't2':t2}, [], ['t1', 't2'])
+        pytest.raises(InvalidTask, my_runner.run_tasks, dispatcher)
+
+    def test_task_not_picklabe_thread(self, reporter, dep_manager):
+        def creator():
+            return {'basename': 't2', 'actions': [lambda: True]}
+        t1 = Task("t1", [(my_print, ["out a"] )] )
+        t2 = Task("t2", None, loader=DelayedLoader(creator, executed='t1'))
+        my_runner = runner.MThreadRunner(dep_manager, reporter)
+        dispatcher = TaskDispatcher({'t1':t1, 't2':t2}, [], ['t1', 't2'])
+        # threaded code have no problems with closures
+        my_runner.run_tasks(dispatcher)
+        assert ('start', t1) == reporter.log.pop(0), reporter.log
+        assert ('execute', t1) == reporter.log.pop(0)
+        assert ('success', t1) == reporter.log.pop(0)
+        assert ('start', t2) == reporter.log.pop(0)
+        assert ('execute', t2) == reporter.log.pop(0)
+        assert ('success', t2) == reporter.log.pop(0)
+
 
 
 @pytest.mark.skipif('not runner.MRunner.available()')
 class TestMRunner_execute_task(object):
-    def test_hold(self, reporter, depfile):
-        run = runner.MRunner(Dependency, depfile.name, reporter)
+    def test_hold(self, reporter, dep_manager):
+        run = runner.MRunner(dep_manager, reporter)
         task_q = Queue()
-        task_q.put(runner.Hold()) # to test
+        task_q.put(runner.JobHold()) # to test
         task_q.put(None) # to terminate function
         result_q = Queue()
         run.execute_task_subprocess(task_q, result_q)
         run.finish()
         # nothing was done
-        assert result_q.empty() # pragma: no cover (coverage bug?)
+        assert result_q.empty()
+
+    def test_full_task(self, reporter, dep_manager):
+        # test execute_task_subprocess can receive a full Task object
+        run = runner.MRunner(dep_manager, reporter)
+        t1 = Task('t1', [simple_result])
+        task_q = Queue()
+        task_q.put(runner.JobTask(t1)) # to test
+        task_q.put(None) # to terminate function
+        result_q = Queue()
+        run.execute_task_subprocess(task_q, result_q)
+        run.finish()
+        # check result
+        assert result_q.get() == {'name': 't1', 'reporter': 'execute_task'}
+        assert result_q.get()['task']['result'] == 'my-result'
+        assert result_q.empty()
+
 
 
 def test_MThreadRunner_available():
