@@ -37,6 +37,27 @@ class TestTaskCheckInput(object):
 
 
 
+class TestTaskCompare(object):
+    def test_equal(self):
+        # only task name is used to compare for equality
+        t1 = task.Task("foo", None)
+        t2 = task.Task("bar", None)
+        t3 = task.Task("foo", None)
+        assert t1 != t2
+        assert t1 == t3
+
+
+    def test_lt(self):
+        # task name is used to compare/sort tasks
+        t1 = task.Task("foo", None)
+        t2 = task.Task("bar", None)
+        t3 = task.Task("gee", None)
+        assert t1 > t2
+        sorted_names = sorted(t.name for t in (t1,t2,t3))
+        assert sorted_names == ['bar', 'foo', 'gee']
+
+
+
 class TestTaskInit(object):
 
     def test_groupTask(self):
@@ -58,10 +79,11 @@ class TestTaskInit(object):
         # when task is created, options contain the default values
         p1 = {'name':'p1', 'default':'p1-default'}
         p2 = {'name':'p2', 'default':'', 'short':'m'}
-        t = task.Task("MyName", None, params=[p1, p2])
-        t._init_options()
+        t = task.Task("MyName", None, params=[p1, p2], pos_arg='pos')
         assert 'p1-default' == t.options['p1']
         assert '' == t.options['p2']
+        assert 'pos' == t.pos_arg
+        assert None == t.pos_arg_val # always unitialized
 
     def test_setup(self):
         t = task.Task("task5", ['action'], setup=["task2"])
@@ -105,6 +127,10 @@ class TestTaskUpToDate(object):
                       uptodate=[(custom_check, [123], {'xxx':'yyy'})])
         assert t.uptodate[0] == (custom_check, [123], {'xxx':'yyy'})
 
+    def test_str(self):
+        t = task.Task("Task X", ["taskcmd"], uptodate=['my-cmd xxx'])
+        assert t.uptodate[0] == ('my-cmd xxx', [], {})
+
     def test_object_with_configure(self):
         class Check(object):
             def __call__(self): return True
@@ -162,12 +188,22 @@ class TestTaskDeps(object):
         assert [(None, None, None), (True, None, None)] == my_task.uptodate
 
 
+
+class TestTask_Loader(object):
+    def test_delayed_after_execution(self):
+        # after `executed` creates an implicit task_dep
+        delayed = task.DelayedLoader(lambda: None, executed='foo')
+        t1 = task.Task('bar', None, loader=delayed)
+        assert t1.task_dep == ['foo']
+
+
 class TestTask_Getargs(object):
     def test_ok(self):
         getargs = {'x' : ('t1','x'), 'y': ('t2','z')}
         t = task.Task('t3', None, getargs=getargs)
-        assert 't1' in t.setup_tasks
-        assert 't2' in t.setup_tasks
+        assert len(t.uptodate) == 2
+        assert ['t1', 't2'] == sorted([t.uptodate[0][0].dep_name,
+                                       t.uptodate[1][0].dep_name])
 
     def test_invalid_desc(self):
         getargs = {'x' : 't1'}
@@ -430,6 +466,17 @@ class TestTaskDoc(object):
         assert "" == t.doc
 
 
+class TestTaskUpdateFromPickle(object):
+    def test_change_value(self):
+        t = task.Task("my_name", ["action"])
+        assert {} == t.values
+        class FakePickle():
+            def __init__(self):
+                self.values = [1,2,3]
+        t.update_from_pickle(FakePickle().__dict__)
+        assert [1,2,3] == t.values
+        assert 'my_name' == t.name
+
 class TestDictToTask(object):
     def testDictOkMinimum(self):
         dict_ = {'name':'simple','actions':['xpto 14']}
@@ -441,3 +488,72 @@ class TestDictToTask(object):
 
     def testDictMissingFieldAction(self):
         pytest.raises(action.InvalidTask, task.dict_to_task, {'name':'xpto 14'})
+
+
+
+class TestResultDep(object):
+    def test_single(self, depfile):
+        dep_manager = depfile
+
+        tasks = {'t1': task.Task("t1", None, uptodate=[task.result_dep('t2')]),
+                 't2': task.Task("t2", None),
+                 }
+        # _config_task was executed and t2 added as task_dep
+        assert ['t2'] == tasks['t1'].setup_tasks
+
+        # first t2 result
+        tasks['t2'].result = 'yes'
+        dep_manager.save_success(tasks['t2'])
+        assert 'run' == dep_manager.get_status(tasks['t1'], tasks)  # first time
+
+        tasks['t1'].save_extra_values()
+        dep_manager.save_success(tasks['t1'])
+        assert 'up-to-date' == dep_manager.get_status(tasks['t1'], tasks)
+
+        # t2 result changed
+        tasks['t2'].result = '222'
+        dep_manager.save_success(tasks['t2'])
+
+        tasks['t1'].save_extra_values()
+        dep_manager.save_success(tasks['t1'])
+        assert 'run' == dep_manager.get_status(tasks['t1'], tasks)
+
+        tasks['t1'].save_extra_values()
+        dep_manager.save_success(tasks['t1'])
+        assert 'up-to-date' == dep_manager.get_status(tasks['t1'], tasks)
+
+
+    def test_group(self, depfile):
+        dep_manager = depfile
+
+        tasks = {'t1': task.Task("t1", None, uptodate=[task.result_dep('t2')]),
+                 't2': task.Task("t2", None, task_dep=['t2:a', 't2:b'],
+                                 has_subtask=True),
+                 't2:a': task.Task("t2:a", None),
+                 't2:b': task.Task("t2:b", None),
+                 }
+        # _config_task was executed and t2 added as task_dep
+        assert ['t2'] == tasks['t1'].setup_tasks
+
+        # first t2 result
+        tasks['t2:a'].result = 'yes1'
+        dep_manager.save_success(tasks['t2:a'])
+        tasks['t2:b'].result = 'yes2'
+        dep_manager.save_success(tasks['t2:b'])
+        assert 'run' == dep_manager.get_status(tasks['t1'], tasks)  # first time
+
+        tasks['t1'].save_extra_values()
+        dep_manager.save_success(tasks['t1'])
+        assert 'up-to-date' == dep_manager.get_status(tasks['t1'], tasks)
+
+        # t2 result changed
+        tasks['t2:a'].result = '222'
+        dep_manager.save_success(tasks['t2:a'])
+
+        tasks['t1'].save_extra_values()
+        dep_manager.save_success(tasks['t1'])
+        assert 'run' == dep_manager.get_status(tasks['t1'], tasks)
+
+        tasks['t1'].save_extra_values()
+        dep_manager.save_success(tasks['t1'])
+        assert 'up-to-date' == dep_manager.get_status(tasks['t1'], tasks)
