@@ -3,6 +3,7 @@
 import os
 import datetime
 import operator
+import six
 
 import pytest
 
@@ -11,16 +12,28 @@ from doit import tools
 from doit import task
 
 
-def test_create_folder():
-    def rm_dir():
-        if os.path.exists(DIR_DEP):
-            os.removedirs(DIR_DEP)
+class TestCreateFolder(object):
+    def test_create_folder(self):
+        def rm_dir():
+            if os.path.exists(DIR_DEP):
+                os.removedirs(DIR_DEP)
 
-    DIR_DEP = os.path.join(os.path.dirname(__file__),"parent/child/")
-    rm_dir()
-    assert True == tools.create_folder(DIR_DEP)
-    assert os.path.exists(DIR_DEP)
-    rm_dir()
+        DIR_DEP = os.path.join(os.path.dirname(__file__),"parent/child/")
+        rm_dir()
+        tools.create_folder(DIR_DEP)
+        assert os.path.exists(DIR_DEP)
+        rm_dir()
+
+    def test_error_if_path_is_a_file(self):
+        def rm_file(path):
+            if os.path.exists(path):
+                os.remove(path)
+
+        path = os.path.join(os.path.dirname(__file__), "test_create_folder")
+        with open(path, 'w') as fp:
+            fp.write('testing')
+        pytest.raises(OSError, tools.create_folder, path)
+        rm_file(path)
 
 
 class TestTitleWithActions(object):
@@ -42,75 +55,6 @@ class TestRunOnce(object):
         assert True == tools.run_once(t, t.values)
 
 
-class TestResultDep(object):
-    def test_single(self, depfile):
-        dep_manager = depfile
-
-        tasks = {'t1': task.Task("t1", None, uptodate=[tools.result_dep('t2')]),
-                 't2': task.Task("t2", None),
-                 }
-        # _config_task was executed and t2 added as task_dep
-        assert ['t2'] == tasks['t1'].task_dep
-
-        # first t2 result
-        tasks['t2'].result = 'yes'
-        dep_manager.save_success(tasks['t2'])
-        assert 'run' == dep_manager.get_status(tasks['t1'], tasks)  # first time
-
-        tasks['t1'].save_extra_values()
-        dep_manager.save_success(tasks['t1'])
-        assert 'up-to-date' == dep_manager.get_status(tasks['t1'], tasks)
-
-        # t2 result changed
-        tasks['t2'].result = '222'
-        dep_manager.save_success(tasks['t2'])
-
-        tasks['t1'].save_extra_values()
-        dep_manager.save_success(tasks['t1'])
-        assert 'run' == dep_manager.get_status(tasks['t1'], tasks)
-
-        tasks['t1'].save_extra_values()
-        dep_manager.save_success(tasks['t1'])
-        assert 'up-to-date' == dep_manager.get_status(tasks['t1'], tasks)
-
-
-    def test_group(self, depfile):
-        dep_manager = depfile
-
-        tasks = {'t1': task.Task("t1", None, uptodate=[tools.result_dep('t2')]),
-                 't2': task.Task("t2", None, task_dep=['t2:a', 't2:b'],
-                                 has_subtask=True),
-                 't2:a': task.Task("t2:a", None),
-                 't2:b': task.Task("t2:b", None),
-                 }
-        # _config_task was executed and t2 added as task_dep
-        assert ['t2'] == tasks['t1'].task_dep
-
-        # first t2 result
-        tasks['t2:a'].result = 'yes1'
-        dep_manager.save_success(tasks['t2:a'])
-        tasks['t2:b'].result = 'yes2'
-        dep_manager.save_success(tasks['t2:b'])
-        assert 'run' == dep_manager.get_status(tasks['t1'], tasks)  # first time
-
-        tasks['t1'].save_extra_values()
-        dep_manager.save_success(tasks['t1'])
-        assert 'up-to-date' == dep_manager.get_status(tasks['t1'], tasks)
-
-        # t2 result changed
-        tasks['t2:a'].result = '222'
-        dep_manager.save_success(tasks['t2:a'])
-
-        tasks['t1'].save_extra_values()
-        dep_manager.save_success(tasks['t1'])
-        assert 'run' == dep_manager.get_status(tasks['t1'], tasks)
-
-        tasks['t1'].save_extra_values()
-        dep_manager.save_success(tasks['t1'])
-        assert 'up-to-date' == dep_manager.get_status(tasks['t1'], tasks)
-
-
-
 class TestConfigChanged(object):
     def test_invalid_type(self):
         class NotValid(object):pass
@@ -128,7 +72,7 @@ class TestConfigChanged(object):
         assert False == ub(t1, t1.values)
 
     def test_unicode(self):
-        ua = tools.config_changed({'x':u"中文"})
+        ua = tools.config_changed({'x':six.u("中文")})
         ub = tools.config_changed('b')
         t1 = task.Task("TaskX", None, uptodate=[ua])
         assert False == ua(t1, t1.values)
@@ -248,7 +192,7 @@ class TestCheckTimestampUnchanged(object):
         assert True == check(t, t.values)
 
         # stored timestamp less than current, up to date
-        future_time = t.values.values()[0] + 100
+        future_time = list(six.itervalues(t.values))[0] + 100
         monkeypatch.setattr(check, '_get_time', lambda: future_time)
         assert False == check(t, t.values)
 
@@ -272,16 +216,16 @@ class TestCheckTimestampUnchanged(object):
         assert check_a._key != check_m._key
 
 
-class TestInteractiveAction(object):
+class TestLongRunning(object):
     def test_success(self):
         TEST_PATH = os.path.dirname(__file__)
         PROGRAM = "python %s/sample_process.py" % TEST_PATH
-        my_action = tools.InteractiveAction(PROGRAM + " please fail")
+        my_action = tools.LongRunning(PROGRAM + " please fail")
         got = my_action.execute()
         assert got is None
 
     def test_ignore_keyboard_interrupt(self, monkeypatch):
-        my_action = tools.InteractiveAction('')
+        my_action = tools.LongRunning('')
         class FakeRaiseInterruptProcess(object):
             def __init__(self, *args, **kwargs):
                 pass
@@ -291,6 +235,21 @@ class TestInteractiveAction(object):
         got = my_action.execute()
         assert got is None
 
+class TestInteractive(object):
+    def test_fail(self):
+        TEST_PATH = os.path.dirname(__file__)
+        PROGRAM = "python %s/sample_process.py" % TEST_PATH
+        my_action = tools.Interactive(PROGRAM + " please fail")
+        got = my_action.execute()
+        assert isinstance(got, exceptions.TaskFailed)
+
+    def test_success(self):
+        TEST_PATH = os.path.dirname(__file__)
+        PROGRAM = "python %s/sample_process.py" % TEST_PATH
+        my_action = tools.Interactive(PROGRAM + " ok")
+        got = my_action.execute()
+        assert got is None
+
 
 class TestPythonInteractiveAction(object):
     def test_success(self):
@@ -304,3 +263,18 @@ class TestPythonInteractiveAction(object):
         my_action = tools.PythonInteractiveAction(raise_x)
         got = my_action.execute()
         assert isinstance(got, exceptions.TaskError)
+
+    def test_returned_dict_saved_result_values(self):
+        def val(): return {'x': 3}
+        my_action = tools.PythonInteractiveAction(val)
+        got = my_action.execute()
+        assert got is None
+        assert my_action.result == {'x': 3}
+        assert my_action.values == {'x': 3}
+
+    def test_returned_string_saved_result(self):
+        def val(): return 'hello'
+        my_action = tools.PythonInteractiveAction(val)
+        got = my_action.execute()
+        assert got is None
+        assert my_action.result == 'hello'
